{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: Childwin.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: CHILDWIN.PAS,v 1.7 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit Childwin;

interface

uses
  Windows, Classes, Graphics, Forms, Controls, StdCtrls, SynEdit, ExtCtrls,
  SynEditHighlighter, SynHighlighterPerl, sysUtils, iniFiles,
  SynEditTypes, Messages, breakpoints, Buttons, SynEditPrintPreview,
  SynEditPrint, ComCtrls, Menus, DockPanel;

type
  TMDIChild = class;
  
  TBreakpointPlugin = class(TSynEditPlugin)
  private
    F_mdi: TMDIChild;
  protected
    procedure AfterPaint(ACanvas: TCanvas; AClip: TRect;
      FirstLine, LastLine: integer); override;
    procedure LinesInserted(FirstLine, Count: integer); override;
    procedure LinesDeleted(FirstLine, Count: integer); override;
  public
    constructor Create(mdi: TMDIChild);
  end;


  TMDIChild = class(TForm)
    pnlChild: TPanel;
    SynEdit: TSynEdit;
    pnlPrintPreview: TPanel;
    pnlPrintPreviewNavigation: TPanel;
    SynEditPrint: TSynEditPrint;
    pnlPrintPreviewNavigationRight: TPanel;
    chbLineNumbers: TCheckBox;
    pnlClient: TPanel;
    spbtnClose: TSpeedButton;
    spbtnPrint: TSpeedButton;
    spbtnLast: TSpeedButton;
    spbtnNextPage: TSpeedButton;
    spbtnPreviousPage: TSpeedButton;
    spbtnFirst: TSpeedButton;
    Label1: TLabel;
    SynEditPrintPreview: TSynEditPrintPreview;
    edtZoom: TEdit;
    UpDown1: TUpDown;
    lbPage: TLabel;
    ppmChild: TPopupMenu;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    Close1: TMenuItem;
    Preferences1: TMenuItem;
    N3: TMenuItem;
    PrintPreview1: TMenuItem;
    Open1: TMenuItem;
    N4: TMenuItem;
    New1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SynEditSpecialLineColors(Sender: TObject; Line: Integer;
      var Special: Boolean; var FG, BG: TColor);
    procedure SynEditGutterClick(Sender: TObject; X, Y, Line: Integer;
      mark: TSynEditMark);
    procedure SynEditMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SynEditClick(Sender: TObject);
    procedure SynEditChange(Sender: TObject);
    procedure SynEditStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SynEditMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SynEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SynEditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure chbLineNumbersClick(Sender: TObject);
    procedure spbtnFirstClick(Sender: TObject);
    procedure spbtnPreviousPageClick(Sender: TObject);
    procedure spbtnNextPageClick(Sender: TObject);
    procedure spbtnLastClick(Sender: TObject);
    procedure spbtnPrintClick(Sender: TObject);
    procedure spbtnCloseClick(Sender: TObject);
    procedure edtZoomChange(Sender: TObject);
    procedure FormGetSiteInfo(Sender: TObject; DockClient: TControl;
      var InfluenceRect: TRect; MousePos: TPoint; var CanDock: Boolean);
    procedure FormDockOver(Sender: TObject; Source: TDragDockObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
  private
    { Private declarations }
    m_slBreakpoints: TStringList;
    m_nCurrentLine: Integer;
    m_nErrorLine: Integer;
    m_sFileName: String;
    m_nFileAge: Integer;
    m_bReadOnly: Boolean;
    m_BreakpointPlugin: TBreakpointPlugin;
    m_bSynEditVisibleBeforeDocking: Boolean;
    m_bPrintPreviewVisibleBeforeDocking: Boolean;
    procedure SetBreakpoints(s: String);
    function IsPointInSelection(Value: TPoint): boolean;
    procedure SetReadOnly(b: Boolean);
    function GetHasChanged: Boolean;
    function GetAsString: String;
    procedure SetAsString(s: String);
    function GetSelectedTextAtCursor: String;
    procedure UpdatePageLabel;
    procedure SetHeaderAndFooter;
    function GetIsDockHost: Boolean;
  public
    { Public declarations }
    procedure CheckFileAge;
    procedure SetCurrentLine(nLine: Integer);
    procedure SetErrorLine(nLine: Integer);
    function SaveFile: String;
    function SaveFileAs(sFileName: String): String;
    procedure OpenFile(sFileName: String);
    procedure AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    procedure DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    constructor Create(AOwner: TComponent); override;
    procedure AfterConstruction; override;
    procedure PrintPreview;
    procedure Print;
    property FileName: String Read m_sFileName Write m_sFileName;
    property Breakpoints: String Write SetBreakpoints;
    property ReadOnly: Boolean Read m_bReadOnly;
    property HasChanged: Boolean Read GetHasChanged;
    property AsString: String Read GetAsString Write SetAsString;
    property SelectedTextAtCursor: String Read GetSelectedTextAtCursor;
    property IsDockHost: Boolean Read GetIsDockHost;
  end;


implementation

uses
  Main, f_Preferences, f_Help, f_Breakpoints, f_HelpIndex;

{$R *.DFM}


////////////////////////////////////////////////////////////////////////////////
//  TBreakpointPlugin = class(TSynEditPlugin)
////////////////////////////////////////////////////////////////////////////////
constructor TBreakpointPlugin.Create(mdi: TMDIChild);
begin
  inherited Create(mdi.SynEdit);
  F_mdi := mdi;
end;


procedure TBreakpointPlugin.AfterPaint(ACanvas: TCanvas; AClip: TRect;
      FirstLine, LastLine: integer);
begin
end;


procedure TBreakpointPlugin.LinesInserted(FirstLine, Count: integer);
var
  i, nLine: Integer;
begin
  frmBreakpoints.eltBreakpoints.IsUpdating := True;
  try
    F_mdi.m_slBreakpoints.Sorted := False;
    for i := F_mdi.m_slBreakpoints.Count - 1 downto 0 do begin
      nLine := StrToIntDef('$' + F_mdi.m_slBreakpoints[i], 0);
      if nLine >= FirstLine then begin
        F_mdi.m_slBreakpoints[i] := IntToHex(nLine + Count, 8);
        frmBreakpoints.MoveBreakpoint(F_mdi, F_mdi.m_sFileName, nLine, nLine + Count, '');
      end;
    end;
  finally
    frmBreakpoints.eltBreakpoints.IsUpdating := False;
    F_mdi.m_slBreakpoints.Duplicates := dupIgnore;
    F_mdi.m_slBreakpoints.Sorted := True;
  end;
end;


procedure TBreakpointPlugin.LinesDeleted(FirstLine, Count: integer);
var
  i, nLine: Integer;
begin
  frmBreakpoints.eltBreakpoints.IsUpdating := True;
  try
    F_mdi.m_slBreakpoints.Sorted := False;
    i := 0;
    while i < F_mdi.m_slBreakpoints.Count do begin
      nLine := StrToIntDef('$' + F_mdi.m_slBreakpoints[i], 0);
      if (FirstLine <= nLine) and (nLine < (FirstLine + Count)) then begin
        // Breakpoint is in range of deleted lines and must be deleted
        F_mdi.m_slBreakpoints.Delete(i);
        frmBreakpoints.DeleteBreakpoint(F_mdi, F_mdi.m_sFileName, nLine, '');
      end else if (nLine >= (FirstLine + Count)) then begin
        // Breakpoint is below range of deleted lines and must be moved
        F_mdi.m_slBreakpoints[i] := IntToHex(nLine - Count, 8);
        frmBreakpoints.MoveBreakpoint(F_mdi, F_mdi.m_sFileName, nLine, nLine - Count, '');
        inc(i);
      end else inc(i);
    end;
  finally
    frmBreakpoints.eltBreakpoints.IsUpdating := False;
    F_mdi.m_slBreakpoints.Duplicates := dupIgnore;
    F_mdi.m_slBreakpoints.Sorted := True;
  end;
end;


////////////////////////////////////////////////////////////////////////////////
//  TMDIChild = class(TForm)
////////////////////////////////////////////////////////////////////////////////
function TMDIChild.GetIsDockHost: Boolean;
begin
  Result := DockClientCount > 0;
end;


procedure TMDIChild.SetHeaderAndFooter;
begin
  with SynEditPrint.Header do begin
    Clear;
    Add(FileName, nil, taLeftJustify, 1);
    Add('$DATE$ $TIME$', nil, taRightJustify, 1);
  end;
  with SynEditPrint.Footer do begin
    Clear;
    Add('Page $PAGENUM$ of $PAGECOUNT$', nil, taRightJustify, 1);
  end;
end;


procedure TMDIChild.UpdatePageLabel;
begin
  lbPage.Caption := Format('Page %d of %d', [SynEditPrintPreview.PageNumber, SynEditPrint.PageCount]);
end;


procedure TMDIChild.Print;
begin
  SynEditPrint.SynEdit := SynEdit;
  SynEditPrint.Title := 'Open Perl IDE - ' + FileName;
  SetHeaderAndFooter;
  MainForm.PrintDialog.FromPage := 1;
  MainForm.PrintDialog.ToPage := SynEditPrint.PageCount;
  MainForm.PrintDialog.MinPage := 0;
  MainForm.PrintDialog.MaxPage := SynEditPrint.PageCount;
  if MainForm.PrintDialog.Execute then begin
    SynEditPrint.PrintRange(MainForm.PrintDialog.FromPage, MainForm.PrintDialog.ToPage);
  end;
end;


procedure TMDIChild.PrintPreview;
begin
  chbLineNumbers.SetFocus;
  SynEditPrint.SynEdit := SynEdit;
  SynEditPrint.Title := 'Open Perl IDE - ' + FileName;
  SetHeaderAndFooter;
  SynEditPrintPreview.UpdatePreview;
  SynEditPrintPreview.FirstPage;
  UpdatePageLabel;
end;


function TMDIChild.GetSelectedTextAtCursor: String;
begin
  if SynEdit.SelAvail
    then Result := SynEdit.SelText
    else Result := SynEdit.GetWordAtRowCol(SynEdit.CaretXY);
end;


function TMDIChild.GetAsString: String;
var
  sl: TStringList;

  function TPointToString(p: TPoint): String;
  begin
    Result := IntToHex(p.x, 8) + IntToHex(p.y, 8);
  end;

begin
  Result := '';
  sl := TStringList.Create;
  try
    sl.Values['Left'] := IntToStr(Left);
    sl.Values['Top'] := IntToStr(Top);
    sl.Values['Width'] := IntToStr(Width);
    sl.Values['Height'] := IntToStr(Height);
    sl.Values['CursorPos'] := TPointToString(SynEdit.CaretXY);
    Result := sl.CommaText;
  finally
    sl.Free;
  end;
end;


procedure TMDIChild.SetAsString(s: String);
var
  sl: TStringList;
  nLeft, nTop, nWidth, nHeight: Integer;
  
  function StringToTPoint(s: String): TPoint;
  begin
    Result.x := StrToIntDef('$' + copy(s, 1, 8), 0);
    Result.y := StrToIntDef('$' + copy(s, 9, 8), 0);
  end;

begin
  sl := TStringList.Create;
  try
    sl.CommaText := s;
    nLeft := StrToIntDef(sl.Values['Left'], 200);
    nTop := StrToIntDef(sl.Values['Top'], 200);
    nWidth := StrToIntDef(sl.Values['Width'], 200);
    nHeight := StrToIntDef(sl.Values['Height'], 200);
    BoundsRect := Rect(nLeft, nTop, nLeft + nWidth, nTop + nHeight);
    SynEdit.CaretXY := StringToTPoint(sl.Values['CursorPos']);
  finally
    sl.Free;
  end;
end;


function TMDIChild.GetHasChanged: Boolean;
begin
  Result := SynEdit.CanUndo;
end;


procedure TMDIChild.SetReadOnly(b: Boolean);
begin
  m_bReadOnly := b;
  if m_bReadOnly then begin
    SynEdit.ReadOnly := True;
    SynEdit.Font.Color := frmPreferences.ReadOnlyEditorMode.ForegroundColor;
    SynEdit.Color := frmPreferences.ReadOnlyEditorMode.BackgroundColor;
  end else begin
    SynEdit.ReadOnly := False;
    SynEdit.Font.Color := frmPreferences.SynEdit.Font.Color;
    SynEdit.Color := frmPreferences.SynEdit.Color;
  end;
end;



procedure TMDIChild.CheckFileAge;
var
  nNewAge: Integer;
begin
  if MainForm.ExecutionState in [ecsDebug, ecsPendingCommand] then exit;
  if m_nFileAge <> -1 then begin
    nNewAge := FileAge(m_sFileName);
    if nNewAge = -1 then begin
      SetReadOnly(False);
      MessageBox(Handle, PChar('File ' + m_sFileName + ' has been deleted.')
        , 'Warning', MB_OK);
      m_nFileAge := -1;
    end else begin
      SetReadOnly((FileGetAttr(m_sFileName) and faReadOnly) <> 0);
      if m_nFileAge <> nNewAge then begin
        if MessageBox(Handle, PChar('File ' + m_sFileName
          + ' has been changed by another appplication.'#13#10
          + 'Do you want to reload it ?') , 'Warning', MB_YESNO) = IDYES then begin
          SynEdit.ClearUndo;
          OpenFile(m_sFileName);
        end else begin
          m_nFileAge := nNewAge;
        end;
      end;
    end;
  end;
end;


constructor TMDIChild.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;


procedure TMDIChild.AfterConstruction;
begin
  inherited AfterConstruction;
  if MainForm.MDIChildCount = 1 then begin
    ShowWindow(Handle, SW_HIDE);
    ShowWindow(Handle, SW_MAXIMIZE);
  end;
end;



procedure TMDIChild.AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
begin
  m_slBreakpoints.Add(IntToHex(nLine, 8));
  SynEdit.InvalidateLine(nLine);
  if Sender = Self
    then MainForm.DispatchBPChangeEvent(Sender, m_sFileName, nLine, '', False);
  if not SynEdit.CanUndo then frmBreakpoints.Breakpoints[m_sFileName].Save;
end;



procedure TMDIChild.DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
var
  idx: Integer;
begin
  idx := m_slBreakpoints.IndexOf(IntToHex(nLine, 8));
  if idx > -1 then m_slBreakpoints.Delete(idx);
  SynEdit.InvalidateLine(nLine);
  if Sender = Self
    then MainForm.DispatchBPChangeEvent(Sender, m_sFileName, nLine, '', True);
  if not SynEdit.CanUndo then frmBreakpoints.Breakpoints[m_sFileName].Save;
end;


procedure TMDIChild.SetBreakpoints(s: String);
var
  i, nLine: Integer;
  bpl: TBreakpointList;
begin
  for i := 0 to m_slBreakpoints.Count - 1 do begin
    nLine := StrToIntDef('$' + m_slBreakpoints[i], 0);
    SynEdit.InvalidateLine(nLine);
  end;
  m_slBreakpoints.Clear;
  bpl := TBreakpointList.Create('temp');
  bpl.AsString := s;
  for i := 0 to bpl.Count - 1 do begin
    m_slBreakpoints.Add(IntToHex(bpl[i].Line, 8));
    SynEdit.InvalidateLine(bpl[i].Line);
  end;
  bpl.Free;
end;


procedure TMDIChild.OpenFile(sFileName: String);
begin
  if sFileName = '' then begin
    m_sFileName := '<new file ' + IntToStr(MainForm.MDIChildCount) + '>';
    Caption := m_sFileName;
  end else begin
    m_sFileName := sFileName;
    m_nFileAge := FileAge(m_sFileName);
    SynEdit.Lines.LoadFromFile(sFileName);
    SetReadOnly((FileGetAttr(m_sFileName) and faReadOnly) <> 0);
    Caption := sFileName;
  end;
end;


function TMDIChild.SaveFileAs(sFileName: String): String;
var
  bSameFileName: Boolean;
  idx: Integer;
  bIsActive: Boolean;
begin
  if (Pos('<new file', sFileName) > 0) or (sFileName = '') then begin
    if Pos('<new file', m_sFileName) > 0
      then MainForm.SaveDialog.FileName := 'noname.pl'
      else MainForm.SaveDialog.FileName := m_sFileName;
    if MainForm.SaveDialog.Execute
      then sFileName := MainForm.SaveDialog.FileName
      else exit;
  end;
  if Pos('<new file', sFileName) = 0 then begin
    bSameFileName := m_sFileName = sFileName;
    m_sFileName := sFileName;
    Caption := m_sFileName;
    SynEdit.Lines.SaveToFile(m_sFileName);
    m_nFileAge := FileAge(m_sFileName);
    SetReadOnly((FileGetAttr(m_sFileName) and faReadOnly) <> 0);
    SynEdit.ClearUndo;
    SynEditStatusChange(nil, []);
    idx := MainForm.tcFiles.Tabs.IndexOfObject(Self);
    bIsActive := idx = MainForm.tcFiles.TabIndex;    
    if idx > -1 then MainForm.tcFiles.Tabs[idx] := ExtractFileName(m_sFileName);
    if bSameFileName and (not SynEdit.CanUndo)
      then frmBreakpoints.Breakpoints[m_sFileName].Save
      else Breakpoints := '';
    if bIsActive and MainForm.IsExecutable(m_sFileName) then MainForm.ChangeProject(m_sFileName);
  end;
  Result := sFileName;
  MainForm.UpdateActions;
end;



function TMDIChild.SaveFile: String;
begin
  Result := SaveFileAs(m_sFileName);
end;



procedure TMDIChild.SetErrorLine(nLine: Integer);
var
  nOldLine: Integer;
begin
  nOldLine := m_nErrorLine;
  m_nErrorLine := nLine;
  if nOldLine <> -1 then SynEdit.InvalidateLine(nOldLine);
  if m_nErrorLine > -1 then begin
    SynEdit.InvalidateLine(m_nErrorLine);
    SynEdit.CaretXY := Point(0, nLine);
    SynEdit.EnsureCursorPosVisible;
  end;
end;


procedure TMDIChild.SetCurrentLine(nLine: Integer);
var
  nOldLine: Integer;
begin
  nOldLine := m_nCurrentLine;
  m_nCurrentLine := nLine;
  if nOldLine <> -1 then SynEdit.InvalidateLine(nOldLine);
  if m_nCurrentLine > -1 then begin
    SynEdit.InvalidateLine(m_nCurrentLine);
    SynEdit.CaretXY := Point(0, nLine);
    SynEdit.EnsureCursorPosVisible;
  end;
end;

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i, idx: Integer;
begin
  if MainForm.LastActiveEditor = Self then MainForm.LastActiveEditor := nil;
  if DockClientCount > 0 then begin
    for i := 0 to DockClientCount - 1 do begin
      if DockClients[i].HostDockSite = Self then begin
        DockClients[i].Visible := False;
        DockClients[i].HostDockSite := nil;
      end;
    end;
  end;
  idx := MainForm.tcFiles.Tabs.IndexOfObject(Self);
  if idx > -1 then begin
    MainForm.tcFiles.Tabs.Delete(idx);
    if MainForm.tcFiles.Tabs.Count = 0 then MainForm.ChangeProject('');
  end;
  Action := caFree;
end;

procedure TMDIChild.FormCreate(Sender: TObject);
begin
  m_bSynEditVisibleBeforeDocking := True;
  m_bPrintPreviewVisibleBeforeDocking := False;
  m_BreakpointPlugin := TBreakpointPlugin.Create(Self);
  m_nFileAge := -1;
  m_nCurrentLine := -1;
  m_nErrorLine := -1;
  m_slBreakpoints := TStringList.Create;
  m_slBreakpoints.Sorted := True;
  m_slBreakpoints.Duplicates := dupIgnore;
end;

procedure TMDIChild.FormDestroy(Sender: TObject);
begin
  DockHandler.UnRegisterDockHost(Self);
  m_slBreakpoints.Free;
  MainForm.UpdateActions;
  m_BreakpointPlugin.Free;
end;

procedure TMDIChild.SynEditSpecialLineColors(Sender: TObject;
  Line: Integer; var Special: Boolean; var FG, BG: TColor);
begin
  if (Line = m_nCurrentLine) or (Line = m_nErrorLine) then begin
    if m_slBreakpoints.IndexOf(IntToHex(Line, 8)) = -1 then begin
      if Line = m_nCurrentLine then begin
        FG := frmPreferences.ExecutionLine.ForegroundColor;
        BG := frmPreferences.ExecutionLine.BackgroundColor;
      end else begin
        FG := frmPreferences.ErrorLine.ForegroundColor;
        BG := frmPreferences.ErrorLine.BackgroundColor;
      end;
    end else begin
      FG := frmPreferences.ExecutionLine.ForegroundColor or frmPreferences.BreakpointLine.ForegroundColor;
      BG := frmPreferences.ExecutionLine.BackgroundColor xor frmPreferences.BreakpointLine.BackgroundColor;
    end;
    Special := True;
  end else begin
    if m_slBreakpoints.IndexOf(IntToHex(Line, 8)) > -1 then begin
      FG := frmPreferences.BreakpointLine.ForegroundColor;
      BG := frmPreferences.BreakpointLine.BackgroundColor;
      Special := True;
    end;
  end;
end;


procedure TMDIChild.SynEditGutterClick(Sender: TObject; X, Y,
  Line: Integer; mark: TSynEditMark);
var
  idx: Integer;
begin
  if Sender = nil then begin
    idx := m_slBreakpoints.IndexOf(IntToHex(Line, 8));
    if (Line > 0) and (Line <= SynEdit.Lines.Count) then begin
      if (idx > -1)
        then DeleteBreakpoint(Self, m_sFileName, Line, '')
        else AddBreakpoint(Self, m_sFileName, Line, '');
    end;
  end;
end;


function TMDIChild.IsPointInSelection(Value: TPoint): boolean;
var
  ptBegin, ptEnd: TPoint;
begin
  with SynEdit do begin
    ptBegin := BlockBegin;
    ptEnd := BlockEnd;
    if (Value.Y >= ptBegin.Y) and (Value.Y <= ptEnd.Y) and
      ((ptBegin.Y <> ptEnd.Y) or (ptBegin.X <> ptEnd.X))
      then begin
      if SelectionMode = smLine then
        Result := TRUE
      else if (SelectionMode = smColumn) then begin
        if (ptBegin.X > ptEnd.X) then
          Result := (Value.X >= ptEnd.X) and (Value.X < ptBegin.X)
        else if (ptBegin.X < ptEnd.X) then
          Result := (Value.X >= ptBegin.X) and (Value.X < ptEnd.X)
        else
          Result := FALSE;
      end else
        Result := ((Value.Y > ptBegin.Y) or (Value.X >= ptBegin.X)) and
          ((Value.Y < ptEnd.Y) or (Value.X < ptEnd.X));
    end else
      Result := FALSE;
  end;
end;



procedure TMDIChild.SynEditMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if MainForm.ExecutionState = ecsDebug then begin
    if SynEdit.SelAvail
    and IsPointInSelection(SynEdit.PixelsToRowColumn(Point(x, y))) then begin
      SynEdit.Hint := SynEdit.SelText;
    end else begin
      SynEdit.Hint := SynEdit.GetWordAtRowCol(SynEdit.PixelsToRowColumn(Point(x, y)))
    end
  end else SynEdit.Hint := '';
end;

procedure TMDIChild.SynEditClick(Sender: TObject);
begin
  SetErrorLine(-1);
end;

procedure TMDIChild.SynEditChange(Sender: TObject);
begin
  SetErrorLine(-1);
end;

procedure TMDIChild.SynEditStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
var
  sOldCaption: String;
begin
  sOldCaption := Caption;
  if SynEdit.CanUndo
    then Caption := m_sFileName + '*'
    else Caption := m_sFileName;
  if sOldCaption <> Caption then MainForm.UpdateActions;
end;

procedure TMDIChild.FormActivate(Sender: TObject);
var
  idx: Integer;
begin
  if pnlChild.Visible then MainForm.LastActiveEditor := Self;
  idx := MainForm.tcFiles.Tabs.IndexOfObject(Self);
  if (idx > -1) and (idx <> MainForm.tcFiles.TabIndex) then begin
    MainForm.tcFiles.TabIndex := idx;
    MainForm.tcFilesChange(nil);
  end;
  // Update settings
  if frmPreferences.chbEnableVirtualSpace.Checked
    then SynEdit.Options := SynEdit.Options + [eoScrollPastEol]
    else SynEdit.Options := SynEdit.Options - [eoScrollPastEol];
  SynEdit.Font.Assign(frmPreferences.SynEdit.Font);
  SynEdit.Gutter.Color := frmPreferences.SynEdit.Gutter.Color;
  SynEdit.RightEdgeColor := frmPreferences.SynEdit.RightEdgeColor;
  SynEdit.SelectedColor.Foreground := frmPreferences.SynEdit.SelectedColor.Foreground;
  SynEdit.SelectedColor.Background := frmPreferences.SynEdit.SelectedColor.Background;
  SynEdit.RightEdge := frmPreferences.SynEdit.RightEdge;
  if SynEdit.ReadOnly then begin
    SynEdit.Font.Color := frmPreferences.ReadOnlyEditorMode.ForegroundColor;
    SynEdit.Color := frmPreferences.ReadOnlyEditorMode.BackgroundColor;
  end else begin
    SynEdit.Color := frmPreferences.SynEdit.Color;
  end;
  SynEdit.TabWidth := frmPreferences.SynEdit.TabWidth;
  // Update actions
  MainForm.UpdateActions;
  // Check FileAge
  CheckFileAge;
end;

procedure TMDIChild.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  nRes: Integer;
begin
  if SynEdit.CanUndo then begin
    nRes := Application.MessageBox(PChar('Save changes to ' + m_sFileName + ' ?'),
      'Open Perl IDE', MB_YESNOCANCEL);
    if nRes = IDYES then SaveFile;
    CanClose := nRes <> IDCANCEL;
  end else CanClose := True;
end;

procedure TMDIChild.SynEditMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  nLine: Integer;
begin
  if (x < SynEdit.Gutter.RealGutterWidth(SynEdit.CharWidth) - 2)
  and (not SynEdit.SelAvail) then begin
    nLine := SynEdit.PixelsToRowColumn(Point(X, Y)).Y;
    SynEditGutterClick(nil, x, y, nLine, nil);
  end;
end;


procedure TMDIChild.SynEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then begin
    frmHelpIndex.ShowIndex(SelectedTextAtCursor);
  end;
end;

procedure TMDIChild.SynEditKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = 90 then MainForm.UpdateActions;
  if Key = VK_DELETE then SynEditStatusChange(nil, []);
end;

procedure TMDIChild.chbLineNumbersClick(Sender: TObject);
begin
  SynEditPrint.LineNumbers := chbLineNumbers.Checked;
  SynEditPrintPreview.UpdatePreview;
  SynEditPrintPreview.Refresh;
end;

procedure TMDIChild.spbtnFirstClick(Sender: TObject);
begin
  SynEditPrintPreview.FirstPage;
  UpdatePageLabel;
end;

procedure TMDIChild.spbtnPreviousPageClick(Sender: TObject);
begin
  SynEditPrintPreview.PreviousPage;
  UpdatePageLabel;
end;

procedure TMDIChild.spbtnNextPageClick(Sender: TObject);
begin
  SynEditPrintPreview.NextPage;
  UpdatePageLabel;
end;

procedure TMDIChild.spbtnLastClick(Sender: TObject);
begin
  SynEditPrintPreview.LastPage;
  UpdatePageLabel;
end;

procedure TMDIChild.spbtnPrintClick(Sender: TObject);
begin
  Print;
end;

procedure TMDIChild.spbtnCloseClick(Sender: TObject);
begin
  pnlPrintPreview.Visible := False;
end;

procedure TMDIChild.edtZoomChange(Sender: TObject);
var
  i, code: Integer;
begin
  val(edtZoom.Text, i, code);
  if code = 0 then begin
    SynEditPrintPreview.ScalePercent := i;
  end;
end;

procedure TMDIChild.FormGetSiteInfo(Sender: TObject; DockClient: TControl;
  var InfluenceRect: TRect; MousePos: TPoint; var CanDock: Boolean);
var
  rct: TRect;
begin
  CanDock := (DockClientCount = 0) and (DockClient is TDockableForm);
  rct := InfluenceRect;
  InflateRect(rct, -(rct.Right - rct.Left) div 3, -(rct.Bottom - rct.Top) div 3);
  CanDock := CanDock and PtInRect(rct, MousePos);
end;

procedure TMDIChild.FormDockOver(Sender: TObject; Source: TDragDockObject;
  X, Y: Integer; State: TDragState; var Accept: Boolean);
var
  mdi: TMDIChild;
  idx: Integer;
  rct: TRect;
begin
  if (Source.Control is TForm) then begin
    mdi := nil;
    idx := MainForm.tcFiles.Tabs.IndexOf('<' + TForm(Source.Control).Caption + '>');
    if idx > -1 then mdi := TMDIChild(MainForm.tcFiles.Tabs.Objects[idx]);
    if mdi = nil then begin
      mdi := MainForm.CreateMDIChild('<' + TForm(Source.Control).Caption + '>', False);
      mdi.pnlChild.Visible := False;
      mdi.UseDockManager := True;
      mdi.DockSite := True;
      DockHandler.RegisterDockHost(mdi);
   end;
  end;
  rct := Source.DockRect;
  InflateRect(rct, -(rct.Right - rct.Left) div 3, -(rct.Bottom - rct.Top) div 3);
  Source.DockRect := rct;
  Accept := UseDockManager;
end;

end.
