{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: FileProcs.pas, released 06 May 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: FileProcs.pas,v 1.2 2002/05/26 13:47:35 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit FileProcs;

interface

uses
  SysUtils, Classes;

type
  TFileProc = procedure(sFileName: String; pParam: Pointer; var bContinue: Boolean) of object;

  IFileFinder = interface ['{E908A180-31BB-11D5-8FDA-DC243B67604A}']
    function GetFiles: TStringList;
    property Files: TStringList Read GetFiles;
  end;

  TFileFinder = class(TInterfacedObject, IFileFinder)
  private
    m_sl: TStringList;
    function GetFiles: TStringList;
  protected
    procedure HandleFile(sFileName: String; pParam: Pointer; var bContinue: Boolean); virtual;
  public
    constructor Create(sFileMask: String; bRecurse: Boolean = True);
    destructor Destroy; override;
    property Files: TStringList Read GetFiles;
  end;


function RecurseDirectories(sFileMask: String; OnFile: TFileProc; pParam: Pointer = nil; bRecurse: Boolean = True): Boolean;
function AddTrailingBackslash(sDirName: String): String;
function RemoveTrailingBackslash(sDirName: String): String;

implementation


function RemoveTrailingBackslash(sDirName: String): String;
begin
  if copy(sDirName, Length(sDirName), 1) = '\'
    then Result := copy(sDirName, 1, Length(sDirName) - 1)
    else Result := sDirName;
end;


function AddTrailingBackslash(sDirName: String): String;
begin
  if copy(sDirName, Length(sDirName), 1) = '\'
    then Result := sDirName
    else Result := sDirName + '\';
end;


////////////////////////////////////////////////////////////////////////////////
//  TFileFinder = class
////////////////////////////////////////////////////////////////////////////////

constructor TFileFinder.Create(sFileMask: String; bRecurse: Boolean = True);
begin
  inherited Create;
  m_sl := TStringList.Create;
  m_sl.Sorted := True;
  m_sl.Duplicates := dupIgnore;
  RecurseDirectories(sFileMask, HandleFile, Pointer(m_sl), bRecurse);
end;


destructor TFileFinder.Destroy;
begin
  m_sl.Free;
  inherited Destroy;
end;


procedure TFileFinder.HandleFile(sFileName: String; pParam: Pointer; var bContinue: Boolean);
begin
  TStringList(pParam).Add(sFileName);
end;


function TFileFinder.GetFiles: TStringList;
begin
  Result := m_sl;
end;



////////////////////////////////////////////////////////////////////////////////

function RecurseDirectories(sFileMask: String; OnFile: TFileProc; pParam: Pointer = nil; bRecurse: Boolean = True): Boolean;

  function InternalHandleDirectory(sDir, sMask: String): Boolean;
  var
    res: Integer;
    sr: TSearchRec;
  begin
    Result := True;

    if bRecurse then begin
      res := FindFirst(sDir + '*.*', faDirectory, sr);
      if res = 0 then begin
        while res = 0 do begin
          if (sr.Name <> '.') and (sr.Name <> '..') then begin
            if (sr.Attr and faDirectory) <> 0 then begin
              Result := InternalHandleDirectory(sDir + sr.Name + '\', sMask);
              if not Result then begin
                FindClose(sr);
                exit;
              end;
            end;
          end;
          res := FindNext(sr);
        end;
        FindClose(sr);
      end;
    end;

    res := FindFirst(sDir + sMask, faArchive, sr);
    if res = 0 then begin
      while res = 0 do begin
        OnFile(sDir + sr.Name, pParam, Result);
        if not Result then begin
          FindClose(sr);
          exit;
        end;
        res := FindNext(sr);
      end;
      FindClose(sr);
    end;
  end;

begin
  Result := False;
  if not Assigned(OnFile) then exit;
  Result := InternalHandleDirectory(ExtractFilePath(sFileMask), ExtractFileName(sFileMask));
end;

end.


