{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_Breakpoints.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_Breakpoints.pas,v 1.7 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_Breakpoints;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, ComCtrls, CommCtrl, breakpoints, ElTree, ElHeader,
  DockPanel;

type
  TBreakpointHasChangedEvent = procedure(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String; bDelete: Boolean) of object;

  TfrmBreakpoints = class(TDockableForm)
    pnlBreakpoints: TPanel;
    Panel2: TPanel;
    eltBreakpoints: TElTree;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure eltBreakpointsResize(Sender: TObject);
    procedure eltBreakpointsHeaderColumnResize(Sender: TObject;
      SectionIndex: Integer);
    procedure eltBreakpointsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure eltBreakpointsDblClick(Sender: TObject);
    procedure eltBreakpointsValidateInplaceEdit(Sender: TObject;
      Item: TElTreeItem; Section: TElHeaderSection; var Text: String;
      var Accept: Boolean);
  private
    { Private declarations }
    m_slBreakpointLists: TStringList;
    procedure DoBreakpointHasChanged(sFileName: String; nLine: Integer; sCondition: String; bDelete: Boolean);
    function GetDataAsString: String;
    procedure SetDataAsString(s: String);
    function GetBreakpoints(sModuleName: String): TBreakpointList;
    function GetCount: Integer;
  public
    { Public declarations }
    procedure Clear;
    procedure MoveBreakpoint(Sender: TObject; sFileName: String; nOldLine, nNewLine: Integer;
      sCondition: String);
    procedure AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    procedure DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    property Breakpoints[sModuleName: String]: TBreakpointList Read GetBreakpoints;
    property DataAsString: String Read GetDataAsString Write SetDataAsString;
    property Count: Integer Read GetCount;
  end;

var
  frmBreakpoints: TfrmBreakpoints;

implementation

uses Main, Childwin;

{$R *.DFM}


function TfrmBreakpoints.GetCount: Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to m_slBreakpointLists.Count - 1 do
    Result := Result + TBreakpointList(m_slBreakpointLists.Objects[i]).Count;
end;


function TfrmBreakpoints.GetBreakpoints(sModuleName: String): TBreakpointList;
var
  idx: Integer;
begin
  idx := m_slBreakpointLists.IndexOf(sModuleName);
  if idx > -1 then begin
    Result := TBreakpointList(m_slBreakpointLists.Objects[idx]);
  end else begin
    Result := TBreakpointList.Create(sModuleName);
    m_slBreakpointLists.AddObject(Result.Name, Result);
  end;
end;


procedure TfrmBreakpoints.Clear;
var
  i: Integer;
begin
  for i := 0 to m_slBreakpointLists.Count - 1 do
    m_slBreakpointLists.Objects[i].Free;
  m_slBreakpointLists.Clear;
  eltBreakpoints.Items.Clear;
end;


function TfrmBreakpoints.GetDataAsString: String;
var
  i: Integer;
  sl: TStringList;
  bpl: TBreakpointList;
begin
  sl := TStringList.Create;
  for i := 0 to m_slBreakpointLists.Count - 1 do begin
    bpl := TBreakpointList(m_slBreakpointLists.Objects[i]);
    if bpl.AsString <> '' then sl.Add(bpl.Name + '=' + bpl.AsString);
  end;
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmBreakpoints.SetDataAsString(s: String);
var
  sl: TStringList;
  i, nBreakpoint: Integer;
  bpl: TBreakpointList;
begin
  eltBreakpoints.IsUpdating := True;
  try
    Clear;
    sl := TStringList.Create;
    sl.CommaText := s;
    for i := 0 to sl.Count - 1 do begin
      bpl := TBreakpointList.Create(sl.Names[i]);
      bpl.AsString := sl.Values[sl.Names[i]];
      m_slBreakpointLists.AddObject(bpl.Name, bpl);
      for nBreakpoint := 0 to bpl.Count - 1 do
        AddBreakpoint(Self, bpl.Name, bpl[nBreakpoint].Line, bpl[nBreakpoint].Condition);
    end;
    sl.Free;
  finally
    eltBreakpoints.IsUpdating := False;
  end;
end;


procedure TfrmBreakpoints.DoBreakpointHasChanged(sFileName: String; nLine: Integer; sCondition: String; bDelete: Boolean);
begin
  MainForm.DispatchBPChangeEvent(Self, sFileName, nLine, sCondition, bDelete);
end;


procedure TfrmBreakpoints.MoveBreakpoint(Sender: TObject; sFileName: String;
  nOldLine, nNewLine: Integer; sCondition: String);
var
  i: Integer;
  ilt: TElTreeItem;
  sOldCondition: String;
  bp: TBreakpoint;
begin
  bp := Breakpoints[sFileName].BreakpointByLine(nOldLine);
  if bp <> nil then begin
    sOldCondition := bp.Condition;
    bp.Free;
    bp := TBreakpoint.Create(Breakpoints[sFileName], nNewLine);
    bp.Condition := sOldCondition;
    for i := 0 to eltBreakpoints.Items.Count - 1 do begin
      ilt := eltBreakpoints.Items[i];
      if (ilt.SubItems[2] = sFileName) and (ilt.SubItems[0] = IntToStr(nOldLine)) then begin
        ilt.SubItems[0] := IntToStr(nNewLine);
        break;
      end;
    end;
  end;
end;



procedure TfrmBreakpoints.DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
var
  i: Integer;
  ilt: TElTreeItem;
begin
  Breakpoints[sFileName].BreakpointByLine(nLine).Free;

  for i := 0 to eltBreakpoints.Items.Count - 1 do begin
    ilt := eltBreakpoints.Items[i];
    if (ilt.SubItems[2] = sFileName) and (ilt.SubItems[0] = IntToStr(nLine)) then begin
      ilt.Delete;
      if Sender = Self then DoBreakpointHasChanged(sFileName, nLine, sCondition, True);
      break;
    end;
  end;

end;


procedure TfrmBreakpoints.AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
var
  ilt: TElTreeItem;
begin
  TBreakpoint.Create(Breakpoints[sFileName], nLine).Condition := sCondition;

  ilt := eltBreakpoints.Items.AddItem(nil);
  ilt.Text := ExtractFileName(sFileName);
  ilt.SubItems.Add(IntToStr(nLine));
  ilt.SubItems.Add(sCondition);
  ilt.SubItems.Add(sFileName);

  if Sender = Self then DoBreakpointHasChanged(sFileName, nLine, sCondition, False);
end;



procedure TfrmBreakpoints.FormCreate(Sender: TObject);
begin
  m_slBreakpointLists := TStringList.Create;
  m_slBreakpointLists.Sorted := True;
end;

procedure TfrmBreakpoints.FormDestroy(Sender: TObject);
begin
  Clear;
  m_slBreakpointLists.Free;
end;

procedure TfrmBreakpoints.eltBreakpointsResize(Sender: TObject);
begin
  with eltBreakpoints do begin
    if IsEditing then EndEdit(True);
    HeaderSections[2].Width := ClientWidth - HeaderSections[1].Width - HeaderSections[0].Width;
  end;
end;

procedure TfrmBreakpoints.eltBreakpointsHeaderColumnResize(Sender: TObject;
  SectionIndex: Integer);
begin
  with eltBreakpoints do begin
    if IsEditing then EndEdit(True);
    HeaderSections[2].Width := ClientWidth - HeaderSections[0].Width - HeaderSections[1].Width;
  end;
end;

procedure TfrmBreakpoints.eltBreakpointsKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  lti: TElTreeItem;
begin
  if (Key = 46) and (eltBreakpoints.ItemFocused <> nil) then begin
    lti := eltBreakpoints.ItemFocused;
    DeleteBreakpoint(Self, lti.SubItems[2], StrToIntDef(lti.SubItems[0], -1), '');
  end;
end;

procedure TfrmBreakpoints.eltBreakpointsDblClick(Sender: TObject);
var
  pt: TPoint;
  nHitColumn: Integer;
  sFileName: String;
  nLine: Integer;
  it: TElTreeItem;
  ItemPart: TSTItemPart;
begin
  GetCursorPos(pt);
  pt := eltBreakpoints.ScreenToClient(pt);
  it := eltBreakpoints.GetItemAt(pt.x, pt.y, ItemPart, nHitColumn);
  if (it <> nil) then begin
    sFileName := it.SubItems[2];
    nLine := StrToIntDef(it.SubItems[0], -1);
    if (sFileName <> '') and (nLine > -1)
      then MainForm.ShowFile(sFileName, nLine, True, False);
  end;
end;


procedure TfrmBreakpoints.eltBreakpointsValidateInplaceEdit(
  Sender: TObject; Item: TElTreeItem; Section: TElHeaderSection;
  var Text: String; var Accept: Boolean);
var
  bp: TBreakpoint;
begin
  if not Accept then exit;
  bp := Breakpoints[Item.SubItems[2]].BreakpointByLine(StrToIntDef(Item.SubItems[0], -1));
  if bp <> nil then begin
    bp.Condition := Text;
    DoBreakpointHasChanged(Item.SubItems[2], StrToIntDef(Item.SubItems[0], -1), Text, False);
  end;
end;

end.





