{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_Help.pas, released 06 May 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_Help.pas,v 1.6 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_Help;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, OleCtrls, shdocvw, ComCtrls, ElTree, Menus, Math,
  clProcess, FileCtrl, FileProcs, DockPanel, AppEvnts, ActnList, ActiveX;

type
  PKeywordOffsetStruct = ^TKeywordOffsetStruct;
  TKeywordOffsetStruct = record
    nFileNo: Integer;
    nOffset: Integer;
  end;

  TfrmHelp = class(TDockableForm)
    pnlClient: TPanel;
    pnlLoader: TPanel;
    pnlIndexFilesTop: TPanel;
    lbLoadingIndex: TLabel;
    pnlIndexLoader: TPanel;
    memoIndexLoader: TMemo;
    pnlLoaderCaption: TPanel;
    memoMakeIDX: TMemo;
    ppmHelp: TPopupMenu;
    miStayOnTop: TMenuItem;
    ApplicationEvents1: TApplicationEvents;
    tmrBringToFront: TTimer;
    m_browser: TWebBrowser;
    pnlTop: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure m_browserBeforeNavigate2(Sender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure miStayOnTopClick(Sender: TObject);
    procedure ppmHelpPopup(Sender: TObject);
    procedure ApplicationEvents1Message(var Msg: tagMSG;
      var Handled: Boolean);
    procedure FormStartDock(Sender: TObject;
      var DragObject: TDragDockObject);
    procedure tmrBringToFrontTimer(Sender: TObject);
  private
    { Private declarations }
    m_sBasePath: String;
    m_sKeywordFileName: String;
    m_slKeywordFileStreams: TStringList;
    m_fsKeywordIndex: TFileStream;
    m_sLastURL: String;
    m_LastSenderControl: TWinControl;
    m_LastSenderForm: TDockableForm;
    m_LastDockHost: TObject;
    // Keyword index functions
    procedure HandleFile(sFileName: String; pParam: Pointer; var bContinue: Boolean);
    procedure ExecuteIndexCreationScript(sSourceFileMask, sFileName: String);
    procedure ClearKeywordFileStreams;
    procedure CombineKeywordIndexFilesAndCreateOffsetFile;
    procedure InternalBringToFront;
    procedure UpdateMakeIDX;
    procedure InternalBuildKeywordIndex;
  protected
    function GetAsString: String; override;
    procedure SetAsString(s: String); override;
  public
    { Public declarations }
    procedure RequestBringToFront;
    procedure ShowHelpViewer;
    procedure Navigate(sURL: String; SenderForm: TDockableForm = nil; SenderControl: TWinControl = nil);
    function GetLocation(i: Integer): String;
    function Find(s: String; var idx: Integer): Boolean;
    procedure GetKeyLocation(idx: Integer; out sKey, sLocation: String);
    function BuildKeywordIndex: Boolean;
    property KeywordIndex: TFileStream Read m_fsKeywordIndex;
    property LastURL: String Read m_sLastURL;
  end;





var
  frmHelp: TfrmHelp;

implementation

uses
  Main, Childwin, f_Preferences, f_HelpContents, f_HelpIndex;

{$R *.DFM}



procedure TfrmHelp.RequestBringToFront;
begin
  tmrBringToFront.Enabled := True;
end;


function TfrmHelp.GetAsString: String;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.CommaText := inherited GetAsString;
  sl.Values['FormStyle'] := IntToStr(ord(FormStyle));
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmHelp.SetAsString(s: String);
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.CommaText := s;
  FormStyle := TFormStyle(StrToIntDef(sl.Values['FormStyle'], ord(fsNormal)));
  sl.Free;
  inherited SetAsString(s);
end;


procedure TfrmHelp.Navigate(sURL: String; SenderForm: TDockableForm = nil; SenderControl: TWinControl = nil);
var
  Flags, TargetFrameName, PostData, Headers: OLEVariant;
  wnd: HWnd;
begin
  m_LastSenderControl := SenderControl;
  m_LastSenderForm := SenderForm;
  InternalBringToFront;
  m_browser.Navigate(sURL, Flags, TargetFrameName, PostData, Headers);
  wnd := m_browser.Handle;
//  windows.SetFocus(wnd);
  wnd := GetWindow(wnd, GW_CHILD);
//  if wnd <> 0 then windows.SetFocus(wnd);
  wnd := GetWindow(wnd, GW_CHILD);
  if wnd <> 0 then windows.SetFocus(wnd);
end;


procedure TfrmHelp.ShowHelpViewer;
begin
  InternalBringToFront;
end;


procedure TfrmHelp.InternalBuildKeywordIndex;
var
  slKeywordIndexFilesFromPreferences: TStringList;
  FileFinder: IFileFinder;
  i, idx, nSize: Integer;
  sFileName: String;
  it: TElTreeItem;
  fs: TFileStream;
begin
  Visible := True;
  pnlLoader.Align := alClient;
  memoIndexLoader.Lines.Clear;
  pnlLoader.Visible := True;
  Cursor := crHourglass;
  Refresh;
  Application.ProcessMessages;
  try
    lbLoadingIndex.Caption := 'Please wait...';
    pnlIndexFilesTop.Refresh;
    memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Keyword index generation started.');
    if m_fsKeywordIndex <> nil then begin
      m_fsKeywordIndex.Free;
      m_fsKeywordIndex := nil;
    end;
    ClearKeywordFileStreams;
    try
      ForceDirectories(ExtractFilePath(m_sKeywordFileName));
      m_fsKeywordIndex := TFileStream.Create(m_sKeywordFileName, fmCreate);
      //
      try
        FileFinder := TFileFinder.Create(m_sBasePath + 'KWI_*.txt', False);
        slKeywordIndexFilesFromPreferences := frmPreferences.KeywordIndexFiles;

        for i := 0 to slKeywordIndexFilesFromPreferences.Count - 1 do begin
          sFileName := slKeywordIndexFilesFromPreferences[i];
          it := TElTreeItem(slKeywordIndexFilesFromPreferences.Objects[i]);
          idx := FileFinder.Files.IndexOf(sFileName);
          // Remove all new files from FileFinder List
          if idx > -1 then FileFinder.Files.Delete(idx);

          try
            fs := TFileStream.Create(sFileName, fmOpenRead);
            nSize := fs.Size;
            fs.Free;
          except
            nSize := 0;
          end;
          // Execute perl script MakeIDX.pl to create keywords
          if nSize = 0 then begin
            memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Extracting keywords for ' + AddTrailingBackslash(it.SubItems[0]));
            ExecuteIndexCreationScript(
              AddTrailingBackslash(it.SubItems[0]) + '*.htm*', sFileName);
          end;
          // File <sFileName> now contains keyword/location pairs.
          fs := TFileStream.Create(sFileName, fmOpenRead);
          m_slKeywordFileStreams.AddObject(sFileName, fs);
        end;

        // Delete all old files
        for i := 0 to FileFinder.Files.Count - 1 do begin
          DeleteFile(FileFinder.Files[i]);
        end;
        memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + IntToStr(FileFinder.Files.Count) + ' old files deleted.');

        CombineKeywordIndexFilesAndCreateOffsetFile;

        memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Keyword index generation finished.');

        frmPreferences.RebuildKeywordIndex := False;

      except
        on E: Exception do begin
          MessageBox(Handle, PChar('Unexpected error during keyword index creation: '
            + m_sKeywordFileName + #13#10 + E.Message
            + #13#10 + 'No keyword index available.'),
            'Open Perl IDE - Help Error', MB_OK);
        end;
      end;
    except
      MessageBox(Handle, PChar('Unable to create offset file '
        + m_sKeywordFileName + #13#10 + 'No keyword index available.'),
        'Open Perl IDE - Help Error', MB_OK);
    end;
  finally
    Cursor := crDefault;
    Visible := False;
    pnlLoader.Visible := False;
  end;
end;


function TfrmHelp.BuildKeywordIndex: Boolean;
var
  sl: TStringList;
  i: Integer;
begin
  try
    if frmPreferences.RebuildKeywordIndex then begin
      InternalBuildKeywordIndex;
    end else begin
      if m_fsKeywordIndex = nil then begin
        try
          sl := frmPreferences.KeywordIndexFiles;
          m_fsKeywordIndex := TFileStream.Create(m_sKeywordFileName, fmOpenRead);
          ClearKeywordFileStreams;
          for i := 0 to sl.Count - 1 do begin
            m_slKeywordFileStreams.AddObject(sl[i], TFileStream.Create(sl[i], fmOpenRead));
          end;
        except
          InternalBuildKeywordIndex;
        end;
      end;
    end;
  finally
    Result := frmPreferences.KeywordIndexFiles.Count > 0;
  end;
end;




procedure TfrmHelp.UpdateMakeIDX;
var
  bUpdate: Boolean;
  sl: TStringList;
  sFileName: String;
begin
  sFileName := m_sBasePath + 'MakeIDX.pl';
  bUpdate := not FileExists(sFileName);
  if not bUpdate then begin
    sl := TStringList.Create;
    try
      sl.LoadFromFile(sFileName);
      bUpdate := sl.Count > 1;
      if not bUpdate then begin
        bUpdate := sl[0] <> memoMakeIDX.Lines[0];
        if bUpdate then bUpdate := Pos('AUTOGENERATED', UpperCase(sl[1])) > 0;
      end;
    finally
      sl.Free;
    end;
  end;
  if bUpdate then begin
    ForceDirectories(m_sBasePath);
    memoMakeIDX.Lines.SaveToFile(sFileName);
  end;
end;



procedure TfrmHelp.InternalBringToFront;
begin
  Visible := True;
  BringToFront;
end;




procedure TfrmHelp.CombineKeywordIndexFilesAndCreateOffsetFile;
var
  i, nFiles, iPos: Integer;
  nOffset: Integer;
  nFileIndex: Integer;
  fs: TFileStream;
  slKeys, slPositions: TStringList;
  sKey, sLocation, sProgress, sOldProgress: String;
  nLength: Integer;
begin
  slPositions := TStringList.Create;
  slKeys := TStringList.Create;
  slKeys.Sorted := True;
  slKeys.Duplicates := dupAccept;
  sOldProgress := '';
  try
    // Combine keys from all keyword index files
    memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Merge keyword files...');
    for nFiles := 0 to m_slKeywordFileStreams.Count - 1 do begin
      memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Add ' + ExtractFileName(m_slKeywordFileStreams[nFiles]));
      fs := TFileStream(m_slKeywordFileStreams.Objects[nFiles]);
      fs.Seek(0, soFromBeginning);
      nOffset := 0;
      while fs.Position < fs.Size do begin
        fs.ReadBuffer(nLength, SizeOf(nLength));
        SetLength(sKey, nLength);
        if nLength > 0 then begin
          fs.ReadBuffer(sKey[1], nLength);
          slKeys.AddObject(sKey, TObject(slPositions.Count));
          slPositions.AddObject(IntToStr(nFiles), TObject(fs.Position - nLength - 4));
        end;
        fs.ReadBuffer(nLength, SizeOf(nLength));
        SetLength(sLocation, nLength);
        if nLength > 0 then begin
          fs.ReadBuffer(sLocation[1], nLength);
        end;
        sProgress := IntToStr(round(100
          * ((nFiles + fs.Position / fs.Size) / m_slKeywordFileStreams.Count)))
          + '%';
        if sProgress <> sOldProgress then begin
          sOldProgress := sProgress;
          lbLoadingIndex.Caption := 'Merge keyword files... ' + sProgress;
          pnlIndexFilesTop.Refresh;
        end;
      end;
    end;
    memoIndexLoader.Lines[memoIndexLoader.Lines.Count - 1] := DateTimeToStr(Now) + ': ' + 'Merge keyword files... finished.';

    pnlLoader.Refresh;

    memoIndexLoader.Lines.Add(DateTimeToStr(Now) + ': ' + 'Create offset file...');
    // Create offset file
    for i := 0 to slKeys.Count - 1 do begin
      iPos := Integer(slKeys.Objects[i]);
      nFileIndex := StrToInt(slPositions[iPos]);
      nOffset := Integer(slPositions.Objects[iPos]);
      m_fsKeywordIndex.WriteBuffer(nFileIndex, SizeOf(nFileIndex));
      m_fsKeywordIndex.WriteBuffer(nOffset, SizeOf(nOffset));
      sProgress := IntToStr(round(100 * (i + 1) / slKeys.Count)) + '%';
      if sProgress <> sOldProgress then begin
        sOldProgress := sProgress;
        lbLoadingIndex.Caption := 'Create offset file... ' + sProgress;
        pnlIndexFilesTop.Refresh;
      end;
    end;
    memoIndexLoader.Lines[memoIndexLoader.Lines.Count - 1] := DateTimeToStr(Now) + ': ' + 'Create offset file... finished.';

  finally
    slKeys.Free;
    slPositions.Free;
  end;
end;


procedure TfrmHelp.ClearKeywordFileStreams;
var
  i: Integer;
begin
  for i := 0 to m_slKeywordFileStreams.Count - 1 do begin
    m_slKeywordFileStreams.Objects[i].Free;
  end;
  m_slKeywordFileStreams.Clear;
end;




procedure TfrmHelp.HandleFile(sFileName: String; pParam: Pointer; var bContinue: Boolean);
var
  s: String;
begin
  if sFileName <> '' then begin
    s := sFileName + #13#10;
    TFileStream(pParam).Write(s[1], Length(s));
  end;
end;


procedure TfrmHelp.ExecuteIndexCreationScript(sSourceFileMask, sFileName: String);
var
  fs: TFileStream;
  clp: TCLProcess;
  sPath: String;
begin
  // Perl script MakeIDX.pl reads files from .fil and writes keywords/location
  // pairs to .txt
  ForceDirectories(ExtractFilePath(sFileName));
  fs := TFileStream.Create(ChangeFileExt(sFileName, '.fil'), fmCreate);
  try
    RecurseDirectories(sSourceFileMask, HandleFile, Pointer(fs));
  finally
    fs.Free;
  end;

  UpdateMakeIDX;

  if frmPreferences.GetPathToPerl(sPath) then begin
    clp := TCLProcess.Create('"' + sPath + 'perl.exe"'
      + '-I. MakeIDX.pl ' + ExtractFileName(ChangeFileExt(sFileName, '')),
      ExtractFilePath(sFileName), nil);
    WaitForSingleObject(clp.ProcessHandle, INFINITE);
  end;

end;




procedure TfrmHelp.GetKeyLocation(idx: Integer; out sKey, sLocation: String);
var
  nLength: Integer;
  fs: TFileStream;
  kwo: TKeywordOffsetStruct;
begin
  m_fsKeywordIndex.Seek(idx * 8, soFromBeginning);
  m_fsKeywordIndex.ReadBuffer(kwo, SizeOf(kwo));
  fs := TFileStream(m_slKeywordFileStreams.Objects[kwo.nFileNo]);
  fs.Seek(kwo.nOffset, soFromBeginning);
  fs.ReadBuffer(nLength, SizeOf(nLength));
  SetLength(sKey, nLength);
  if nLength > 0 then fs.ReadBuffer(sKey[1], nLength);
  fs.ReadBuffer(nLength, SizeOf(nLength));
  SetLength(sLocation, nLength);
  if nLength > 0 then fs.ReadBuffer(sLocation[1], nLength);
end;


function TfrmHelp.Find(s: String; var idx: Integer): Boolean;
var
  i1, i2: Integer;
  sKey, sLocation: String;
  nRes: Integer;
begin
  idx := -1;
  sKey := '';
  i1 := 0;
  i2 := (m_fsKeywordIndex.Size div 8) - 1;
  while (i1 <= i2) do begin
    idx := (i1 + i2) div 2;
    GetKeyLocation(idx, sKey, sLocation);
    nRes := AnsiCompareStr(s, sKey);
    if nRes = 0 then break;
    if nRes < 0 then begin
      i2 := idx - 1;
    end else begin
      i1 := idx + 1;
    end;
  end;
  Result := s = sKey;
  // Find the first matching entry
  if Result then begin
    while s = sKey do begin
      dec(idx);
      GetKeyLocation(idx, sKey, sLocation);
    end;
    inc(idx);
  end;
end;



function TfrmHelp.GetLocation(i: Integer): String;
var
  sKey: String;
begin
  GetKeyLocation(i, sKey, Result);
end;



procedure TfrmHelp.FormCreate(Sender: TObject);
begin
  m_sLastURL := '';
  m_slKeywordFileStreams := TStringList.Create;
  m_sBasePath := ExtractFilePath(Application.ExeName) + 'Help\';
  m_sKeywordFileName := m_sBasePath + 'KeywordIndex.ofs';
end;


procedure TfrmHelp.FormDestroy(Sender: TObject);
begin
  if m_fsKeywordIndex <> nil then m_fsKeywordIndex.Free;
  ClearKeywordFileStreams;
  m_slKeywordFileStreams.Free;
end;

procedure TfrmHelp.FormEndDock(Sender, Target: TObject; X, Y: Integer);
begin
  if (m_LastDockHost = nil) and (Target = nil) then exit;
  if m_sLastURL <> '' then Navigate(m_sLastURL);
end;


procedure TfrmHelp.m_browserBeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
begin
  m_sLastURL := URL;
end;

procedure TfrmHelp.miStayOnTopClick(Sender: TObject);
begin
  miStayOnTop.Checked := not miStayOnTop.Checked;
  if miStayOnTop.Checked then begin
    FormStyle := fsStayOnTop;
  end else begin
    FormStyle := fsNormal;
  end;
  if LastURL <> '' then Navigate(LastURL);
end;

procedure TfrmHelp.ppmHelpPopup(Sender: TObject);
begin
  miStayOnTop.Checked := FormStyle = fsStayOnTop;
end;


procedure TfrmHelp.ApplicationEvents1Message(var Msg: tagMSG;
  var Handled: Boolean);
const
  CGID_IWebBrowser: TGuid = (D1: $ED016940; D2: $BD5B; D3: $11cf; D4: ($BA, $4E, $00, $C0, $4F, $D7, $08, $16));
  HTMLID_FIND = 1;
  HTMLID_VIEWSOURCE = 2;
  HTMLID_OPTIONS = 3;
var
  wnd1, wnd2, wnd3: HWnd;
begin
  wnd1 := m_browser.Handle;
  wnd2:= GetWindow(wnd1, GW_CHILD);
  wnd3:= GetWindow(wnd2, GW_CHILD);
  if (Msg.hwnd <> 0)
  and ((wnd1 = Msg.hwnd) or (wnd2 = Msg.hwnd) or (wnd3 = Msg.hwnd)) then begin
    if (Msg.message = WM_CHAR) then begin
      case Msg.wParam of
        3: begin
          m_browser.ExecWB(OLECMDID_COPY, OLECMDEXECOPT_DODEFAULT, EmptyParam, EmptyParam);
          Handled := True;
        end;
        6: begin
          try
            (m_browser.Document as IOLECommandTarget).Exec(@CGID_IWebBrowser, HTMLID_FIND, 0, EmptyParam, EmptyParam);
          except end;
          Handled := True;
        end;
        27: begin
          if HostDockSite = nil then Hide;
          if m_LastSenderForm <> nil then m_LastSenderForm.Visible := True;
          if m_LastSenderControl <> nil then m_LastSenderControl.SetFocus;
        end;
      end;
    end;
  end;
end;

procedure TfrmHelp.FormStartDock(Sender: TObject;
  var DragObject: TDragDockObject);
begin
  m_LastDockHost := HostDockSite;
end;

procedure TfrmHelp.tmrBringToFrontTimer(Sender: TObject);
begin
  tmrBringToFront.Enabled := False;
  if Visible then BringToFront;
end;


end.




