{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_HelpContents.pas, released 23 Nov 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_HelpContents.pas,v 1.3 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_HelpContents;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ElTree, ExtCtrls, FileProcs, shlObj, activeX, comObj, comCtrls, DockPanel;

type
  TfrmHelpContents = class(TDockableForm)
    pnlContents: TPanel;
    eltHelp: TElTree;
    procedure eltHelpItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure eltHelpItemCollapse(Sender: TObject; Item: TElTreeItem);
    procedure eltHelpClick(Sender: TObject);
  private
    { Private declarations }
    function GetPathName(item: TElTreeItem): String;
    procedure ScanDirectory(sDirectory: String; out sFolders, sFiles: String);
    function CheckContentConfiguration: Boolean;
  public
    { Public declarations }
    procedure InternalBringToFront;
    procedure InternalBuildContentTree;
    procedure ShowContent;
  end;

var
  frmHelpContents: TfrmHelpContents;

implementation

uses
  Main, f_Help, f_Preferences;

{$R *.DFM}



procedure TfrmHelpContents.ShowContent;
begin
  if not CheckContentConfiguration then begin
    Visible := False;
    exit;
  end;
  InternalBringToFront;
  InternalBuildContentTree;
end;



function TfrmHelpContents.CheckContentConfiguration: Boolean;

  function CheckPreferences: Boolean;
  var
    i: Integer;
  begin
    Result := False;
    for i := 0 to frmPreferences.eltHelp.Items.Count - 1 do begin
      Result := frmPreferences.eltHelp.Items[i].SubItems[1] <> 'False';
      if Result then break;
    end;
  end;

begin
  Result := CheckPreferences;
  while (not Result) do begin
    if MessageBox(Handle,
      PChar('There is no help content.'
      + #13#10 + 'To create one, you must open the Preferences | Help - Panel'
      + #13#10 + 'and set Contents to True for at minimum one Help-Location.'
      + #13#10
      + #13#10 + 'Do you want to configure the Help System now ?'),
        'Help System Error', MB_YESNO) = ID_YES then begin
      frmPreferences.pcPreferences.ActivePage := frmPreferences.tsHelp;
      if MainForm.ShowPreferences then begin
        Application.ProcessMessages;
        Result := CheckPreferences;
      end else exit;
    end else exit;
  end;
end;



procedure TfrmHelpContents.InternalBringToFront;
begin
  Visible := True;
end;


procedure TfrmHelpContents.InternalBuildContentTree;
var
  i: Integer;
  it: TElTreeItem;
begin
  if not frmPreferences.RebuildContents and (eltHelp.Items.Count > 0) then exit;
  eltHelp.Items.Clear;
  for i := 0 to frmPreferences.eltHelp.Items.Count - 1 do begin
    if frmPreferences.eltHelp.Items[i].SubItems[1] <> 'False' then begin
      it := eltHelp.Items.AddItem(nil);
      it.Text := frmPreferences.eltHelp.Items[i].Text;
      it.SubItems.Add(frmPreferences.eltHelp.Items[i].SubItems[0]);
      it.ForceButtons := True;
      it.ImageIndex := 36;
    end;
  end;
  frmPreferences.RebuildContents := False;
end;


procedure TfrmHelpContents.ScanDirectory(sDirectory: String; out sFolders, sFiles: String);
var
  sr: TSearchRec;
  slFolders, slFiles: TStringList;
  res: Integer;
  link: IShellLink;
  pf: IPersistFile;
  buf: array[0..2047] of char;
  wfd: WIN32_FIND_DATA;
  ws: WideString;

  function ResolveLink(s: String): String;
  begin
    if LowerCase(ExtractFileExt(s)) = '.lnk' then begin
      ws := s;
      pf.Load(PWideChar(ws), STGM_READ);
      link.Resolve(MainForm.Handle, SLR_NO_UI);
      link.GetPath(@buf[0], SizeOf(buf), wfd, SLGP_UNCPRIORITY);
      Result := PChar(@buf[0]);
    end else begin
      Result := s;
    end;
  end;

begin
  link := CreateComObject(CLSID_ShellLink) as IShellLink;
  pf := link as IPersistFile;
  slFolders := TStringList.Create;
  slFiles := TStringList.Create;
  res := FindFirst(sDirectory + '*.*', faDirectory or faAnyFile, sr);
  if res = 0 then begin
    while res = 0 do begin
      if (sr.Name <> '.') and (sr.Name <> '..') then begin
        if sr.Attr and faDirectory <> 0
          then slFolders.Add(sr.Name)
          else slFiles.Add(ResolveLink(sDirectory + sr.Name));
      end;
      res := FindNext(sr);
    end;
    FindClose(sr);
  end;
  sFolders := slFolders.CommaText;
  sFiles := slFiles.CommaText;
  slFiles.Free;
  slFolders.Free;
end;



function TfrmHelpContents.GetPathName(item: TElTreeItem): String;
begin
  Result := '';
  while item <> nil do begin
    if item.SubItems.Count > 0
      then Result := AddTrailingBackslash(item.SubItems[0]) + Result
      else Result := item.Text + Result;
    item := item.Parent;
  end;
end;


procedure TfrmHelpContents.eltHelpItemExpand(Sender: TObject;
  Item: TElTreeItem);
var
  sPath: String;
  sFolders, sFiles: String;
  sl: TStringList;
  ti: TElTreeItem;
  i: Integer;
begin
  if Item.ImageIndex = 38 then exit;
  Item.ImageIndex := 37;
  Item.Clear;
  sPath := GetPathName(Item);
  Screen.Cursor := crHourGlass;
  try
    ScanDirectory(sPath, sFolders, sFiles);
    sl := TStringList.Create;
    sl.Sorted := True;
    sl.CommaText := sFolders;
    for i := 0 to sl.Count - 1 do begin
      ti := frmHelpContents.eltHelp.Items.AddChild(Item, sl[i]);
      ti.ImageIndex := 36;
      ti.ForceButtons := True;
      ti.SubItems.Add(sl[i]);
    end;
    sl.CommaText := sFiles;
    for i := 0 to sl.Count - 1 do begin
      ti := frmHelpContents.eltHelp.Items.AddChild(Item, ExtractFileName(sl[i]));
      ti.ImageIndex := 38;
      ti.SubItems.Add(sl[i]);
    end;
    sl.Free;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmHelpContents.eltHelpItemCollapse(Sender: TObject;
  Item: TElTreeItem);
begin
  Item.ImageIndex := 36;
end;

procedure TfrmHelpContents.eltHelpClick(Sender: TObject);
var
  pt: TPoint;
  ItemPart: TSTItemPart;
  nHitColumn: Integer;
  it: TElTreeItem;
  sFileName: String;
begin
  GetCursorPos(pt);
  pt := frmHelpContents.eltHelp.ScreenToClient(pt);
  it := frmHelpContents.eltHelp.GetItemAt(pt.x, pt.y, ItemPart, nHitColumn);
  if (it <> nil) and (it = frmHelpContents.eltHelp.Selected) then begin
    if it.ImageIndex = 38 then begin
      sFileName := it.SubItems[0];
      frmHelp.Navigate(sFileName, Self, eltHelp);
    end;
  end;
end;

end.
