{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_HelpIndex.pas, released 23 Nov 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_HelpIndex.pas,v 1.3 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_HelpIndex;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, FileProcs, Math, comCtrls, DockPanel;

type
  TfrmHelpIndex = class(TDockableForm)
    Panel1: TPanel;
    pnlKeywordIndex: TPanel;
    lbxKeywords: TListBox;
    ScrollBar1: TScrollBar;
    Panel2: TPanel;
    edtKeyword: TEdit;
    Panel3: TPanel;
    procedure lbxKeywordsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edtKeywordChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lbxKeywordsClick(Sender: TObject);
    procedure ScrollBar1Scroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure edtKeywordKeyPress(Sender: TObject; var Key: Char);
    procedure edtKeywordKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lbxKeywordsDblClick(Sender: TObject);
    procedure lbxKeywordsKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    m_nSelectedItem: Integer;
    m_nBaseItem: Integer;
    m_nCurrentPosInIndexList: Integer;
    m_bInternalChange: Boolean;
    function CheckIndexConfiguration: Boolean;
    procedure ShowKeyword(s: String; nItemIndex: Integer = -1);
  public
    { Public declarations }
    procedure ShowIndex(sKeyword: String = '');
    procedure InternalBringToFront;
    procedure ShowList(nPos: Integer = -1; bForceVisible: Boolean = False);
    property SelectedItem: Integer Read m_nSelectedItem Write m_nSelectedItem;
    property BaseItem: Integer Read m_nBaseItem;
  end;

var
  frmHelpIndex: TfrmHelpIndex;

implementation

uses
  f_Preferences, f_Help, Main, ChildWin;

{$R *.DFM}


procedure TfrmHelpIndex.ShowIndex(sKeyword: String = '');
begin
  if sKeyword <> '' then ShowKeyword(sKeyword) else ShowList(-1, True);
end;



procedure TfrmHelpIndex.InternalBringToFront;
begin
  Visible := True;
(*
  if HostDockSite = nil then begin
    BringToFront;
  end else if HostDockSite is TPageControl then begin
    TPageControl(HostDockSite).ActivePage := TTabSheet(Parent);
  end else begin
    HostDockSite.BringToFront;
  end;
  Visible := True;
*)
end;



function TfrmHelpIndex.CheckIndexConfiguration: Boolean;

  function CheckPreferences: Boolean;
  begin
    Result := (frmPreferences.KeywordIndexFiles.Count > 0);
  end;

begin
  Result := CheckPreferences;
  while (not Result) do begin
    if MessageBox(Handle,
      PChar('There is no keyword file.'
      + #13#10 + 'To create one, you must open the Preferences | Help - Panel'
      + #13#10 + 'and activate Keyword Index Generation for at minimum one Help-Location.'
      + #13#10
      + #13#10 + 'Do you want to configure the Help System now ?'),
        'Help System Error', MB_YESNO) = ID_YES then begin
      frmPreferences.pcPreferences.ActivePage := frmPreferences.tsHelp;
      if MainForm.ShowPreferences then begin
        Application.ProcessMessages;
        Result := CheckPreferences;
      end else exit;
    end else exit;
  end;
end;


procedure TfrmHelpIndex.ShowKeyword(s: String; nItemIndex: Integer = -1);
var
  idx: Integer;
  bFound: Boolean;
begin
  if not CheckIndexConfiguration then begin
    Visible := False;
    exit;
  end;
  if not frmHelp.BuildKeywordIndex then begin
    frmHelp.Visible := False;
    Visible := False;
    exit;
  end;
  Visible := True;

  Cursor := crHourGlass;
  try
    if nItemIndex = -1 then begin
      bFound := frmHelp.Find(s, idx);
      edtKeyword.Text := s;
      lbxKeywords.ItemIndex := -1;
      SelectedItem := idx;
      ShowList(idx);
      if bFound then begin
        frmHelp.Navigate(frmHelp.GetLocation(idx), Self, edtKeyword);
      end else begin
        edtKeyword.SetFocus;
      end;
    end else begin
      frmHelp.Navigate(frmHelp.GetLocation(frmHelpIndex.BaseItem + nItemIndex),
        Self, edtKeyword);
    end;
  finally
    Cursor := crDefault;
  end;
end;



procedure TfrmHelpIndex.ShowList(nPos: Integer = -1; bForceVisible: Boolean = False);
var
  i: Integer;
  nItemCount: Integer;
  lb: Integer;
  sKey, sLocation: String;
begin
  if nPos = -1 then nPos := m_nCurrentPosInIndexList;

  if not CheckIndexConfiguration then begin
    Visible := False;
    exit;
  end;
  if not frmHelp.BuildKeywordIndex then begin
    frmHelp.Visible := False;
    Visible := False;
    exit;
  end;
  if bForceVisible then Visible := True;

  m_nCurrentPosInIndexList := nPos;
  if lbxKeywords.ItemIndex > -1
    then m_nSelectedItem := m_nBaseItem + lbxKeywords.ItemIndex;
  lbxKeywords.DoubleBuffered := True;
  nItemCount := (lbxKeywords.ClientHeight div lbxKeywords.ItemHeight);
  Scrollbar1.Position := max(0, nPos);
  ScrollBar1.PageSize := nItemCount;
  ScrollBar1.LargeChange := nItemCount;
  Scrollbar1.Min := nItemCount div 2;
  Scrollbar1.Max := (frmHelp.KeywordIndex.Size div 8) - 1 + nItemCount div 2;
  lbxKeywords.Items.Clear;
  lb := max(0, nPos - nItemCount div 2);
  lb := min(lb, (frmHelp.KeywordIndex.Size div 8) - nItemCount);

  for i := lb to lb + nItemCount - 1 do begin
    frmHelp.GetKeyLocation(i, sKey, sLocation);
    lbxKeywords.Items.Add(sKey);
  end;

  m_nBaseItem := lb;
  if (m_nSelectedItem > -1) and (m_nSelectedItem - m_nBaseItem < nItemCount) then begin
    lbxKeywords.ItemIndex := max(-1, m_nSelectedItem - m_nBaseItem);
  end else begin
    lbxKeywords.ItemIndex := -1;
  end;

end;


procedure TfrmHelpIndex.lbxKeywordsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if lbxKeywords.ItemIndex > -1
    then m_nSelectedItem := m_nBaseItem + lbxKeywords.ItemIndex
    else m_nSelectedItem := -1;
  case Key of
    VK_DOWN: begin
      if lbxKeywords.ItemIndex = lbxKeywords.Items.Count - 1 then begin
//        if m_nBaseItem + lbxKeywords.Items.Count < m_slKey.Count then begin
        if m_nBaseItem + lbxKeywords.Items.Count < (frmHelp.KeywordIndex.Size div 8) then begin
          ScrollBar1.Position := ScrollBar1.Position + 1;
          lbxKeywords.ItemIndex := -1;
          ShowList(ScrollBar1.Position);
        end;
      end;
    end;
    VK_UP: begin
      if lbxKeywords.ItemIndex = 0 then begin
        ScrollBar1.Position := ScrollBar1.Position - 1;
        lbxKeywords.ItemIndex := -1;
        ShowList(ScrollBar1.Position);
      end;
    end;
    VK_PRIOR: begin
      Scrollbar1.Position := max(Scrollbar1.Min, Scrollbar1.Position - Scrollbar1.PageSize);
      ShowList(ScrollBar1.Position);
    end;
    VK_NEXT: begin
      Scrollbar1.Position := min(Scrollbar1.Max, Scrollbar1.Position + Scrollbar1.PageSize);
      ShowList(ScrollBar1.Position);
    end;
    VK_END: begin
      Scrollbar1.Position := Scrollbar1.Max;
      ShowList(ScrollBar1.Position);
    end;
    VK_HOME: begin
      Scrollbar1.Position := Scrollbar1.Min;
      ShowList(ScrollBar1.Position);
    end;
  else
  end;
end;

procedure TfrmHelpIndex.edtKeywordChange(Sender: TObject);
var
  idx: Integer;
  bRightSelection: Boolean;
  sKey, sLocation: String;
begin
  if m_bInternalChange then exit;
  frmHelp.Find(edtKeyword.Text, idx);
  bRightSelection := lbxKeyWords.ItemIndex > -1;
  if bRightSelection then begin
    bRightSelection := idx > -1;
    if bRightSelection then begin
      frmHelp.GetKeyLocation(idx, sKey, sLocation);
      bRightSelection := lbxKeywords.Items[lbxKeywords.ItemIndex] = sKey;
    end;
  end;
  if not bRightSelection then begin
    lbxKeywords.ItemIndex := -1;
    m_nSelectedItem := idx;
    ShowList(idx);
  end;
end;

procedure TfrmHelpIndex.FormCreate(Sender: TObject);
begin
  m_nSelectedItem := -1;
  m_nBaseItem := -1;
end;

procedure TfrmHelpIndex.lbxKeywordsClick(Sender: TObject);
begin
  if lbxKeywords.ItemIndex > -1 then begin
    m_bInternalChange := True;
    try frmHelpIndex.edtKeyword.Text := lbxKeywords.Items[lbxKeywords.ItemIndex]; except end;
    m_bInternalChange := False;
  end;
end;

procedure TfrmHelpIndex.ScrollBar1Scroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  if m_nBaseItem + lbxKeywords.Items.Count <= (frmHelp.KeywordIndex.Size div 8) then begin
    ShowList(ScrollPos);
  end else dec(ScrollPos);
end;

procedure TfrmHelpIndex.edtKeywordKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then lbxKeywordsDblClick(nil);
  if Key = #27 then begin
    if Floating then begin
      Hide;
    end else begin
      if MainForm.LastActiveEditor <> nil then begin
        ReleaseCapture;
        MainForm.LastActiveEditor.BringToFront;
        windows.SetFocus(MainForm.LastActiveEditor.SynEdit.Handle);
      end;
    end;
  end;
end;

procedure TfrmHelpIndex.edtKeywordKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key in [VK_DOWN, VK_UP, VK_PRIOR, VK_NEXT] then begin
    SendMessage(lbxKeywords.Handle, WM_KEYDOWN, Key, 0);
    Key := 0;
  end;
end;

procedure TfrmHelpIndex.lbxKeywordsDblClick(Sender: TObject);
begin
  if lbxKeywords.ItemIndex > -1 then begin
    ShowKeyword(lbxKeywords.Items[lbxKeywords.ItemIndex], lbxKeywords.ItemIndex);
  end;
end;

procedure TfrmHelpIndex.lbxKeywordsKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = #13 then begin
    if lbxKeywords.ItemIndex > -1 then begin
      ShowKeyword(lbxKeywords.Items[lbxKeywords.ItemIndex], lbxKeywords.ItemIndex);
    end;
  end;
  if Key = #27 then begin
    if Floating then begin
      Hide;
    end else begin
      if MainForm.ActiveMDIChild <> nil then begin
        ReleaseCapture;
        windows.SetFocus(TMDIChild(MainForm.ActiveMDIChild).SynEdit.Handle);
      end;
    end;
  end;
end;

procedure TfrmHelpIndex.FormResize(Sender: TObject);
begin
  if frmHelp.KeywordIndex <> nil then ShowList;
end;

end.
