{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_Preferences.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_Preferences.pas,v 1.7 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_Preferences;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, SynEdit, SynEditHighlighter,
  Menus, ElTree, FileCtrl, SynEditMiscClasses, typInfo;

type
  IFontsAndColorsControl = interface
    function GetFont: TFont;
    function GetBackgroundColor: TColor;
    procedure SetBackgroundColor(c: TColor);
    function GetForegroundColor: TColor;
    procedure SetForegroundColor(c: TColor);
    // properties
    property Font: TFont Read GetFont;
    property BackgroundColor: TColor Read GetBackgroundColor Write SetBackgroundColor;
    property ForegroundColor: TColor Read GetForegroundColor Write SetForegroundColor;
  end;

  TFontsAndColorsControl = class(TInterfacedObject, IFontsAndColorsControl)
  private
    m_obj: TObject;
    function GetFont: TFont;
    function GetBackgroundColor: TColor;
    procedure SetBackgroundColor(c: TColor);
    function GetForegroundColor: TColor;
    procedure SetForegroundColor(c: TColor);
  protected
    constructor Create(obj: TObject);
  end;

  TSpecialLine = class
  private
    m_colForeground, m_colBackground: TColor;
  public
    property ForegroundColor: TColor Read m_colForeground Write m_colForeground;
    property BackgroundColor: TColor Read m_colBackground Write m_colBackground;
  end;

  TPassiveConsole = class
  private
    m_colForeground: TColor;
  public
    property ForegroundColor: TColor Read m_colForeground Write m_colForeground;
  end;

  TRightEdge = class
  end;

  TReadOnlyEditorMode = class
  private
    m_colForeground: TColor;
    m_colBackground: TColor;
  public
    property ForegroundColor: TColor Read m_colForeground Write m_colForeground;
    property BackgroundColor: TColor Read m_colBackground Write m_colBackground;
  end;


  TSelection = class
  end;


  TfrmPreferences = class(TForm)
    pnlBottom: TPanel;
    pnlClient: TPanel;
    pcPreferences: TPageControl;
    tsGeneral: TTabSheet;
    btnOK: TButton;
    btnCancel: TButton;
    tsCodeInside: TTabSheet;
    cld: TColorDialog;
    fntDlg: TFontDialog;
    tsHelp: TTabSheet;
    gbxHelp: TGroupBox;
    ppmHelp: TPopupMenu;
    miAddHelpLocation: TMenuItem;
    miDeleteHelpLocation: TMenuItem;
    pnlHelpDirectories: TPanel;
    Label3: TLabel;
    pnlHelpConfiguration: TPanel;
    eltHelp: TElTree;
    tsSyntaxHighlighting: TTabSheet;
    tsFontsAndColors: TTabSheet;
    GroupBox1: TGroupBox;
    pnlSyntaxHighlighting: TPanel;
    Label14: TLabel;
    btnResetToDefault: TButton;
    pnlSepEditorOptions: TPanel;
    pnlEditorOptionsExample: TPanel;
    pnlEditorExample: TPanel;
    SynEdit: TSynEdit;
    gbxFontAndColors: TGroupBox;
    GroupBox3: TGroupBox;
    Label7: TLabel;
    Label16: TLabel;
    pnlForegroundColor: TPanel;
    pnlBackgroundColor: TPanel;
    btnForegroundColor: TButton;
    btnBackgroundColor: TButton;
    btnClearForeground: TButton;
    btnClearBackground: TButton;
    chbBold: TCheckBox;
    chbItalic: TCheckBox;
    chbUnderline: TCheckBox;
    chbStrikeout: TCheckBox;
    Panel5: TPanel;
    Label15: TLabel;
    pnlFontAndColorWindows: TPanel;
    eltFontsAndColors: TElTree;
    pnlFontsAndColors_Element: TPanel;
    Panel1: TPanel;
    lbFontsAndColorsItem_Caption: TLabel;
    Panel6: TPanel;
    pnlFontsAndColorPreview: TPanel;
    pnlFontsAndColorPreviewTop: TPanel;
    Label6: TLabel;
    Panel4: TPanel;
    pnlFontsAndColors_Preview: TPanel;
    Panel9: TPanel;
    lbxElements: TListBox;
    pnlFontsAndColors_FontStyle: TPanel;
    chbFontsAndColors_FontBold: TCheckBox;
    chbFontsAndColors_FontItalic: TCheckBox;
    chbFontsAndColors_FontUnderline: TCheckBox;
    chbFontsAndColors_FontStrikeout: TCheckBox;
    pnlFontsAndColors_Foreground: TPanel;
    Label18: TLabel;
    pnlFontsAndColors_FontColor: TPanel;
    btnFontsAndColors_ForegroundColor: TButton;
    pnlFontAndColors_Font: TPanel;
    cbxFontsAndColors_FontName: TComboBox;
    cbxFontsAndColors_FontSize: TComboBox;
    Label17: TLabel;
    Label19: TLabel;
    pnlFontsAndColors_Backgound: TPanel;
    Label5: TLabel;
    pnlFontsAndColors_BackgroundColor: TPanel;
    btnFontsAndColors_BackgroundColor: TButton;
    btnResetFontsAndColorsToDefault: TButton;
    gbxGeneral: TGroupBox;
    pnlDirectories: TPanel;
    Label12: TLabel;
    Label13: TLabel;
    edtPathToPerl: TEdit;
    btnSearchPathToPerl: TButton;
    Panel10: TPanel;
    pnlGeneralSep1: TPanel;
    pnlEditorOptions: TPanel;
    Label2: TLabel;
    Label9: TLabel;
    chbEnableVirtualSpace: TCheckBox;
    chbFindTextAtCursor: TCheckBox;
    edtTabWidth: TEdit;
    Panel7: TPanel;
    pnlDebug2: TPanel;
    Label8: TLabel;
    Label10: TLabel;
    cbxDebugDesktop: TComboBox;
    chbEvaluateHint: TCheckBox;
    pnlGeneralEnvironment2: TPanel;
    Label1: TLabel;
    Label4: TLabel;
    Label11: TLabel;
    cbxDefaultDesktop: TComboBox;
    cbxRunDesktop: TComboBox;
    Label20: TLabel;
    Label21: TLabel;
    edtConsoleWindowBuffer: TEdit;
    edtErrorWindowBuffer: TEdit;
    udConsoleWindowBuffer: TUpDown;
    udErrorWindowBuffer: TUpDown;
    udTabWidth: TUpDown;
    pnlNotSupported: TPanel;
    Memo1: TMemo;
    Label22: TLabel;
    edtRightEdge: TEdit;
    udRightEdge: TUpDown;
    procedure btnSearchPathToPerlClick(Sender: TObject);
    procedure edtPathToPerlKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure lbxElementsClick(Sender: TObject);
    procedure chbEnableVirtualSpaceClick(Sender: TObject);
    procedure chbStyleClick(Sender: TObject);
    procedure btnForegroundColorClick(Sender: TObject);
    procedure btnBackgroundColorClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnResetToDefaultClick(Sender: TObject);
    procedure btnClearForegroundClick(Sender: TObject);
    procedure btnClearBackgroundClick(Sender: TObject);
    procedure eltHelpDblClick(Sender: TObject);
    procedure miAddHelpLocationClick(Sender: TObject);
    procedure miDeleteHelpLocationClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure eltHelpDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure eltHelpDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure eltFontsAndColorsKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure chbFontsAndColors_FontBoldClick(Sender: TObject);
    procedure cbxFontsAndColors_FontSizeChange(Sender: TObject);
    procedure btnFontsAndColors_ForegroundColorClick(Sender: TObject);
    procedure btnFontsAndColors_BackgroundColorClick(Sender: TObject);
    procedure tsFontsAndColorsShow(Sender: TObject);
    procedure eltFontsAndColorsMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btnResetFontsAndColorsToDefaultClick(Sender: TObject);
    procedure edtTabWidthChange(Sender: TObject);
    procedure edtRightEdgeChange(Sender: TObject);
  private
    { Private declarations }
    m_sha: TSynHighlighterAttributes;
    m_sHighlighterDefault: String;
    m_sBasePath: String;
    m_bRebuildKeywordIndex: Boolean;
    m_bRebuildContents: Boolean;
    m_slKeywordIndexFiles: TStringList;
    m_CurrentFontsAndColorsControl: IFontsAndColorsControl;
    m_bLoadFontsAndColorsItemSettings: Boolean;
    m_ErrorLine: TSpecialLine;
    m_BreakpointLine: TSpecialLine;
    m_ExecutionLine: TSpecialLine;
    m_PassiveConsole: TPassiveConsole;
    m_ReadOnlyEditorMode: TReadOnlyEditorMode;
    m_RightEdge: TRightEdge;
    m_Selection: TSelection;
    // Fonts and Colors Default Values
    m_sFontsAndColorsDefault: String;
    //
    function InternalGetFontsAndColorsAsString(obj: TObject): String;
    procedure InternalSetFontsAndColorsAsString(obj: TObject; s: String);
    function GetAttributesAsString(sName: String): String;
    procedure SetAttributesAsString(sName: String; s: String);
    //
    function GetEnvironmentAsString: String;
    procedure SetEnvironmentAsString(s: String);
    function GetDirectoriesAsString: String;
    procedure SetDirectoriesAsString(s: String);
    function GetHighlighterAsString: String;
    procedure SetHighlighterAsString(s: String);
    function GetFontsAndColorsAsString: String;
    procedure SetFontsAndColorsAsString(s: String);

    //
    procedure CheckPath;
    function AddBackslash(s: String): String;
    //
    procedure ShowHighlighterAttributes;
    //
    function GetNewFileName(sBase, sExt: String): String;
    //
    function GetKeywordIndexFiles: TStringList;
    procedure SetRebuildKeywordIndex(b: Boolean);
    procedure SetRebuildContents(b: Boolean);
    // Fonts and Colors
    procedure ShowFontsAndColorsItem(it: TElTreeItem);
    procedure ShowFontsAndColorsItem_Settings;
    procedure ShowFontsAndColorsItem_Preview;
    function GetFontAsString(AFont: TFont): String;
    procedure SetFontAsString(AFont: TFont; s: String);
    procedure ApplyFontsAndColorsItemSettings;
    // Window buffer size
    function GetConsoleWindowBuffer: Integer;
    procedure SetConsoleWindowBuffer(n: Integer);
    function GetErrorWindowBuffer: Integer;
    procedure SetErrorWindowBuffer(n: Integer);
    function GetRightEdge: Integer;
    function GetTabWidth: Integer;
    procedure SetRightEdge(const Value: Integer);
    procedure SetTabWith(const Value: Integer);
  public
    { Public declarations }
    function GetPathToPerl(var sPath: String): Boolean;
    property EnvironmentAsString: String Read GetEnvironmentAsString Write SetEnvironmentAsString;
    property DirectoriesAsString: String Read GetDirectoriesAsString Write SetDirectoriesAsString;
    property HighlighterAsString: String Read GetHighlighterAsString Write SetHighlighterAsString;
    property FontsAndColorsAsString: String Read GetFontsAndColorsAsString Write SetFontsAndColorsAsString;
    property KeywordIndexFiles: TStringList Read GetKeywordIndexFiles;
    property RebuildKeywordIndex: Boolean Read m_bRebuildKeywordIndex Write SetRebuildKeywordIndex;
    property RebuildContents: Boolean Read m_bRebuildContents Write SetRebuildContents;
    property BreakpointLine: TSpecialLine Read m_BreakpointLine;
    property ErrorLine: TSpecialLine Read m_ErrorLine;
    property ExecutionLine: TSpecialLine Read m_ExecutionLine;
    property PassiveConsole: TPassiveConsole Read m_PassiveConsole;
    property ReadOnlyEditorMode: TReadOnlyEditorMode Read m_ReadOnlyEditorMode;
    property ConsoleWindowBuffer: Integer Read GetConsoleWindowBuffer Write SetConsoleWindowBuffer;
    property ErrorWindowBuffer: Integer Read GetErrorWindowBuffer Write SetErrorWindowBuffer;
    property TabWidth: Integer Read GetTabWidth Write SetTabWith;
    property RightEdge: Integer Read GetRightEdge Write SetRightEdge;
  end;




var
  frmPreferences: TfrmPreferences;

implementation

uses Main, f_Breakpoints, f_CallStack, f_Variables, f_Modules,
  f_Console, f_CommunicationLog, f_ErrorOutput, Childwin, f_HelpContents,
  f_HelpIndex;

{$R *.DFM}


type
  TPublFontColorControl = class(TControl)
  published
    property Font;
    property Color;
  end;

////////////////////////////////////////////////////////////////////////////////
//  TFontAndColorControl = class(TInterfacedObject, IFontsAndColorsControl)
////////////////////////////////////////////////////////////////////////////////
constructor TFontsAndColorsControl.Create(obj: TObject);
begin
  inherited Create;
  m_obj := obj;
end;


function TFontsAndColorsControl.GetFont: TFont;
begin
  if m_obj is TControl then begin
    Result := TPublFontColorControl(m_obj).Font
  end else if m_obj is TPassiveConsole then begin
    Result := frmConsole.memoConsole.Font;
  end else begin
    Result := frmPreferences.SynEdit.Font;
  end;
end;


function TFontsAndColorsControl.GetBackgroundColor: TColor;
begin
  if m_obj is TElTree then begin
    Result := TElTree(m_obj).BkColor;
  end else if m_obj is TSynEdit then begin
    Result := TSynEdit(m_obj).Color;
  end else if m_obj is TReadOnlyEditorMode then begin
    Result := TReadOnlyEditorMode(m_obj).BackgroundColor;
  end else if m_obj is TSynGutter then begin
    Result := TSynGutter(m_obj).Color;
  end else if m_obj is TSpecialLine then begin
    Result := TSpecialLine(m_obj).BackgroundColor;
  end else if m_obj is TPassiveConsole then begin
    Result := frmConsole.memoConsole.Color;
  end else if m_obj is TRightEdge then begin
    Result := clNone;
  end else if m_obj is TSelection then begin
    Result := frmPreferences.SynEdit.SelectedColor.Background;
  end else begin
    Result := TPublFontColorControl(m_obj).Color;
  end;
end;


procedure TFontsAndColorsControl.SetBackgroundColor(c: TColor);
begin
  if m_obj is TElTree then begin
    TElTree(m_obj).BkColor := c;
  end else if m_obj is TSynEdit then begin
    TSynEdit(m_obj).Color := c;
  end else if m_obj is TReadOnlyEditorMode then begin
    TReadOnlyEditorMode(m_obj).BackgroundColor := c;
  end else if m_obj is TSynGutter then begin
    TSynGutter(m_obj).Color := c;
  end else if m_obj is TSpecialLine then begin
    TSpecialLine(m_obj).BackgroundColor := c;
    if MainForm.ActiveMDIChild <> nil then begin
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.Invalidate;
    end;
  end else if m_obj is TPassiveConsole then begin
    frmConsole.memoConsole.Color := c;
  end else if m_obj is TRightEdge then begin
  end else if m_obj is TSelection then begin
    frmPreferences.SynEdit.SelectedColor.Background := c;
  end else begin
    TPublFontColorControl(m_obj).Color := c;
  end;
end;


function TFontsAndColorsControl.GetForegroundColor: TColor;
begin
  if m_obj is TElTree then begin
    Result := TElTree(m_obj).TextColor;
  end else if m_obj is TSynGutter then begin
    Result := frmPreferences.SynEdit.Font.Color;
  end else if m_obj is TReadOnlyEditorMode then begin
    Result := TReadOnlyEditorMode(m_obj).ForegroundColor;
  end else if m_obj is TSpecialLine then begin
    Result := TSpecialLine(m_obj).ForegroundColor;
  end else if m_obj is TPassiveConsole then begin
    Result := TPassiveConsole(m_obj).ForegroundColor;
  end else if m_obj is TRightEdge then begin
    Result := frmPreferences.SynEdit.RightEdgeColor;
  end else if m_obj is TSelection then begin
    Result := frmPreferences.SynEdit.SelectedColor.Foreground;
  end else begin
    Result := TPublFontColorControl(m_obj).Font.Color;
  end;
end;


procedure TFontsAndColorsControl.SetForegroundColor(c: TColor);
begin
  if m_obj is TElTree then begin
    TElTree(m_obj).TextColor := c;
  end else if m_obj is TSynGutter then begin
    frmPreferences.SynEdit.Font.Color := c;
  end else if m_obj is TReadOnlyEditorMode then begin
    TReadOnlyEditorMode(m_obj).ForegroundColor := c;
  end else if m_obj is TSpecialLine then begin
    TSpecialLine(m_obj).ForegroundColor := c;
    if MainForm.ActiveMDIChild <> nil then begin
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.Invalidate;
    end;
  end else if m_obj is TPassiveConsole then begin
    TPassiveConsole(m_obj).ForegroundColor := c;
  end else if m_obj is TRightEdge then begin
    frmPreferences.SynEdit.RightEdgeColor := c;
  end else if m_obj is TSelection then begin
    frmPreferences.SynEdit.SelectedColor.Foreground := c;
  end else begin
    TPublFontColorControl(m_obj).Font.Color := c;
  end;
end;


////////////////////////////////////////////////////////////////////////////////
//  TfrmPreferences = class(TForm)
////////////////////////////////////////////////////////////////////////////////
function TfrmPreferences.GetFontsAndColorsAsString: String;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.Values['Editor'] := InternalGetFontsAndColorsAsString(SynEdit);
  sl.Values['Editor_Gutter'] := InternalGetFontsAndColorsAsString(SynEdit.Gutter);
  sl.Values['Editor_ReadOnlyMode'] := InternalGetFontsAndColorsAsString(ReadOnlyEditorMode);
  sl.Values['Editor_RightEdge'] := InternalGetFontsAndColorsAsString(m_RightEdge);
  sl.Values['Editor_Selection'] := InternalGetFontsAndColorsAsString(m_Selection);
  sl.Values['Line_Breakpoint'] := InternalGetFontsAndColorsAsString(BreakpointLine);
  sl.Values['Line_Execution'] := InternalGetFontsAndColorsAsString(ExecutionLine);
  sl.Values['Line_Error'] := InternalGetFontsAndColorsAsString(ErrorLine);
  sl.Values['Breakpoints'] := InternalGetFontsAndColorsAsString(frmBreakpoints.eltBreakpoints);
  sl.Values['Variables'] := InternalGetFontsAndColorsAsString(frmVariables.eltWatches);
  sl.Values['CallStack'] := InternalGetFontsAndColorsAsString(frmCallStack.eltCallStack);
  sl.Values['Modules'] := InternalGetFontsAndColorsAsString(frmModules.eltModules);
  sl.Values['Error'] := InternalGetFontsAndColorsAsString(frmError.memoErrorOutput);
  sl.Values['Console'] := InternalGetFontsAndColorsAsString(frmConsole.memoConsole);
  sl.Values['PassiveConsole'] := InternalGetFontsAndColorsAsString(PassiveConsole);
  sl.Values['StartParameter'] := InternalGetFontsAndColorsAsString(frmConsole.edtParams);
  sl.Values['Contents'] := InternalGetFontsAndColorsAsString(frmHelpContents.eltHelp);
  sl.Values['Index'] := InternalGetFontsAndColorsAsString(frmHelpIndex.edtKeyword);
  sl.Values['Keyword_Listbox'] := InternalGetFontsAndColorsAsString(frmHelpIndex.lbxKeywords);
  sl.Values['Communication_Log'] := InternalGetFontsAndColorsAsString(frmCommunicationLog.memoCommunicationLog);
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetFontsAndColorsAsString(s: String);
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.CommaText := s;
  InternalSetFontsAndColorsAsString(SynEdit, sl.Values['Editor']);
  InternalSetFontsAndColorsAsString(SynEdit.Gutter, sl.Values['Editor_Gutter']);
  InternalSetFontsAndColorsAsString(ReadOnlyEditorMode, sl.Values['Editor_ReadOnlyMode']);
  InternalSetFontsAndColorsAsString(m_RightEdge, sl.Values['Editor_RightEdge']);
  InternalSetFontsAndColorsAsString(m_Selection, sl.Values['Editor_Selection']);
  InternalSetFontsAndColorsAsString(BreakpointLine, sl.Values['Line_Breakpoint']);
  InternalSetFontsAndColorsAsString(ExecutionLine, sl.Values['Line_Execution']);
  InternalSetFontsAndColorsAsString(ErrorLine, sl.Values['Line_Error']);
  InternalSetFontsAndColorsAsString(frmBreakpoints.eltBreakpoints, sl.Values['Breakpoints']);
  InternalSetFontsAndColorsAsString(frmVariables.eltWatches, sl.Values['Variables']);
  InternalSetFontsAndColorsAsString(frmCallStack.eltCallStack, sl.Values['CallStack']);
  InternalSetFontsAndColorsAsString(frmModules.eltModules, sl.Values['Modules']);
  InternalSetFontsAndColorsAsString(frmError.memoErrorOutput, sl.Values['Error']);
  InternalSetFontsAndColorsAsString(frmConsole.memoConsole, sl.Values['Console']);
  InternalSetFontsAndColorsAsString(PassiveConsole, sl.Values['PassiveConsole']);
  InternalSetFontsAndColorsAsString(frmConsole.edtParams, sl.Values['StartParameter']);
  InternalSetFontsAndColorsAsString(frmHelpContents.eltHelp, sl.Values['Contents']);
  InternalSetFontsAndColorsAsString(frmHelpIndex.edtKeyword, sl.Values['Index']);
  InternalSetFontsAndColorsAsString(frmHelpIndex.lbxKeywords, sl.Values['Keyword_Listbox']);
  InternalSetFontsAndColorsAsString(frmCommunicationLog.memoCommunicationLog, sl.Values['Communication_Log']);
  sl.Free;
end;


function TfrmPreferences.GetConsoleWindowBuffer: Integer;
begin
  Result := udConsoleWindowBuffer.Position;
end;


procedure TfrmPreferences.SetConsoleWindowBuffer(n: Integer);
begin
  udConsoleWindowBuffer.Position := n;
end;


function TfrmPreferences.GetErrorWindowBuffer: Integer;
begin
  Result := udErrorWindowBuffer.Position;
end;


procedure TfrmPreferences.SetErrorWindowBuffer(n: Integer);
begin
  udErrorWindowBuffer.Position := n;
end;


procedure TfrmPreferences.SetRebuildKeywordIndex(b: Boolean);
var
  s: String;
begin
  if b then s := 'TRUE' else s := 'FALSE';
  m_bRebuildKeywordIndex := b;
  MainForm.SaveIDESetting('Help', 'RebuildKeywordIndex', s);
end;


procedure TfrmPreferences.SetRebuildContents(b: Boolean);
var
  s: String;
begin
  if b then s := 'TRUE' else s := 'FALSE';
  m_bRebuildContents := b;
  MainForm.SaveIDESetting('Help', 'RebuildContents', s);
end;


function TfrmPreferences.GetKeywordIndexFiles: TStringList;
var
  i: Integer;
  it: TElTreeItem;
begin
  if m_slKeywordIndexFiles <> nil then m_slKeywordIndexFiles.Free;
  m_slKeywordIndexFiles := TStringList.Create;
  for i := 0 to eltHelp.Items.Count - 1 do begin
    it := eltHelp.Items[i];
    if it.SubItems[2] = 'True' then m_slKeywordIndexFiles.AddObject(it.SubItems[4], it);
  end;
  Result := m_slKeywordIndexFiles;
end;


function TfrmPreferences.GetNewFileName(sBase, sExt: String): String;
var
  s: String;
begin
  ForceDirectories(m_sBasePath);
  s := FormatDateTime('yyyymmddhhnnsszzz', Now);
  Result := m_sBasePath + sBase + '_' + s + sExt;
  while FileExists(Result) do begin
    Sleep(1);
    s := FormatDateTime('yyyymmddhhnnsszzz', Now);
    Result := m_sBasePath + sBase + '_' + s + sExt;
  end;
  // Create zero-length file
  with TStringList.Create do begin
    SaveToFile(Result);
    Free;
  end;
end;



function TfrmPreferences.GetAttributesAsString(sName: String): String;
var
  sha: TSynHighlighterAttributes;
  idx: Integer;
  sl: TStringList;

  function BooleanToString(b: Boolean): String;
  begin
    if b then Result := 'TRUE' else Result := 'FALSE';
  end;
  
begin
  sl := TStringList.Create;
  idx := lbxElements.Items.IndexOf(sName);
  if idx > -1 then begin
    sha := TSynHighlighterAttributes(lbxElements.Items.Objects[idx]);
    sl.Add('ForegroundColor=' + ColorToString(sha.Foreground));
    sl.Add('BackgroundColor=' + ColorToString(sha.Background));
    sl.Add('Bold=' + BooleanToString(fsBold in sha.Style));
    sl.Add('Underline=' + BooleanToString(fsUnderline in sha.Style));
    sl.Add('Italic=' + BooleanToString(fsItalic in sha.Style));
    sl.Add('Strikeout=' + BooleanToString(fsStrikeout in sha.Style));
  end;
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetAttributesAsString(sName: String; s: String);
var
  sha: TSynHighlighterAttributes;
  idx: Integer;
  sl: TStringList;

  function StringToBoolean(s: String): Boolean;
  begin
    Result := s <> 'FALSE';
  end;

begin
  idx := lbxElements.Items.IndexOf(sName);
  if idx > -1 then begin
    sha := TSynHighlighterAttributes(lbxElements.Items.Objects[idx]);
    sl := TStringList.Create;
    try
      sl.CommaText := s;
      try sha.Foreground := StringToColor(sl.Values['ForegroundColor']); except end;
      try sha.Background := StringToColor(sl.Values['BackgroundColor']); except end;

      if StringToBoolean(sl.Values['Bold'])
        then sha.Style := sha.Style + [fsBold]
        else sha.Style := sha.Style - [fsBold];
      if StringToBoolean(sl.Values['Underline'])
        then sha.Style := sha.Style + [fsUnderline]
        else sha.Style := sha.Style - [fsUnderline];
      if StringToBoolean(sl.Values['Italic'])
        then sha.Style := sha.Style + [fsItalic]
        else sha.Style := sha.Style - [fsItalic];
      if StringToBoolean(sl.Values['Strikeout'])
        then sha.Style := sha.Style + [fsStrikeout]
        else sha.Style := sha.Style - [fsStrikeout];
    finally
      sl.Free;
    end;
  end;
end;



procedure TfrmPreferences.ShowHighlighterAttributes;
begin
//  pnlAttributes.Visible := m_sha <> nil;
  if m_sha <> nil then begin
    if m_sha.Foreground <> clNone then begin
      pnlForegroundColor.Caption := '';
      pnlForegroundColor.Color := m_sha.Foreground;
    end else begin
      pnlForegroundColor.Caption := '<none>';
      pnlForegroundColor.Color := clBtnFace;
    end;
    if m_sha.Background <> clNone then begin
      pnlBackgroundColor.Caption := '';
      pnlBackgroundColor.Color := m_sha.Background;
    end else begin
      pnlBackgroundColor.Caption := '<none>';
      pnlBackgroundColor.Color := clBtnFace;
    end;
    chbBold.Checked := fsBold in m_sha.Style;
    chbUnderline.Checked := fsUnderline in m_sha.Style;
    chbItalic.Checked := fsItalic in m_sha.Style;
    chbStrikeout.Checked := fsStrikeout in m_sha.Style;
  end;
end;


function TfrmPreferences.AddBackslash(s: String): String;
begin
  Result := s;
  if (Result <> '') and (copy(Result, Length(Result), 1) <> '\')
    then Result := Result + '\';
end;


procedure TfrmPreferences.CheckPath;
var
  sPath: String;
begin
  while not FileExists(AddBackslash(edtPathToPerl.Text) + 'perl.exe') do begin
    if MessageBox(Handle, PChar(AddBackslash(edtPathToPerl.Text)
    + 'perl.exe does not exist.'#13#10
    + 'Do you want to select a new directory ?'),
    'Open Perl IDE', MB_YESNO) <> IDYES then exit;
    if SelectDirectory('Select path to perl.exe', '', sPath) then begin
      edtPathToPerl.Text := AddBackslash(sPath);
    end;
  end;
end;


function TfrmPreferences.GetPathToPerl(var sPath: String): Boolean;
var
  buf: array[0..1023] of char;
  lpszFilePart: PChar;
begin
  Result := FileExists(AddBackslash(edtPathToPerl.Text) + 'perl.exe');
  if Result then begin
    sPath := edtPathToPerl.Text;
  end else begin
    Result := SearchPath(nil, 'perl.exe', nil, SizeOf(buf) - 1, @buf[0], lpszFilePart) > 0;
    if Result then begin
      sPath := ExtractFilePath(buf)
    end else begin
      if not SelectDirectory('Select path to perl.exe', '', sPath) then sPath := '';
    end;
    edtPathToPerl.Text := AddBackslash(sPath);
    CheckPath;
  end;
  sPath := AddBackslash(edtPathToPerl.Text);
  Result := FileExists(sPath + 'perl.exe');
  if Result then begin
    MainForm.ConfigStorage.WriteString('Directories', 'PathToPerl', frmPreferences.edtPathToPerl.Text);
    MainForm.ConfigStorage.Flush;
  end;
end;


function TfrmPreferences.GetHighlighterAsString: String;
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  for i := 0 to lbxElements.Items.Count - 1 do begin
    sl.Add(lbxElements.Items[i] + '=' + GetAttributesAsString(lbxElements.Items[i]));
  end;
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetHighlighterAsString(s: String);
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  sl.CommaText := s;
  for i := 0 to sl.Count - 1 do begin
    SetAttributesAsString(sl.Names[i], sl.Values[sl.Names[i]]);
  end;
  sl.Free;
end;


function TfrmPreferences.GetDirectoriesAsString: String;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.Add('PathToPerl=' + edtPathToPerl.Text);
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetDirectoriesAsString(s: String);
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.CommaText := s;
  edtPathToPerl.Text := sl.Values['PathToPerl'];
  sl.Free;
end;


function TfrmPreferences.GetEnvironmentAsString: String;
var
  sl: TStringList;
begin
// DefaultDesktop=<Desktop name>
// RunDesktop=<Desktop name>
  sl := TStringList.Create;
  sl.Add('DefaultDesktop=' + cbxDefaultDesktop.Text);
  sl.Add('RunDesktop=' + cbxRunDesktop.Text);
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetEnvironmentAsString(s: String);
var
  sl: TStringList;
begin
// DefaultDesktop=<Desktop name>
// RunDesktop=<Desktop name>
  sl := TStringList.Create;
  sl.CommaText := s;
  cbxDefaultDesktop.Text := sl.Values['DefaultDesktop'];
  cbxRunDesktop.Text := sl.Values['RunDesktop'];
  sl.Free;
end;


procedure TfrmPreferences.btnSearchPathToPerlClick(Sender: TObject);
var
  sDirectory: String;
begin
  if SelectDirectory('Select path to perl.exe', '', sDirectory) then begin
    edtPathToPerl.Text := AddBackslash(sDirectory);
    CheckPath;
  end;
end;


procedure TfrmPreferences.edtPathToPerlKeyPress(Sender: TObject;
  var Key: Char);
var
  sPath: String;
begin
  if Key = #13 then begin
    if edtPathToPerl.Text = '' then begin
      GetPathToPerl(sPath);
      edtPathToPerl.Text := AddBackslash(sPath);
    end else begin
      edtPathToPerl.Text := AddBackslash(edtPathToPerl.Text);
    end;
    CheckPath;
  end;
end;


procedure TfrmPreferences.FormShow(Sender: TObject);
begin
  ShowHighlighterAttributes;
  if eltFontsAndColors.Selected <> nil
    then ShowFontsAndColorsItem(eltFontsAndColors.Selected);
end;

procedure TfrmPreferences.lbxElementsClick(Sender: TObject);
var
  idx: Integer;
begin
  idx := lbxElements.ItemIndex;
  if idx > -1
    then m_sha := TSynHighlighterAttributes(lbxElements.Items.Objects[idx])
    else m_sha := nil;
  ShowHighlighterAttributes;
end;


procedure TfrmPreferences.chbEnableVirtualSpaceClick(Sender: TObject);
begin
  if chbEnableVirtualSpace.Checked
    then SynEdit.Options := SynEdit.Options + [eoScrollPastEol]
    else SynEdit.Options := SynEdit.Options - [eoScrollPastEol];
end;

procedure TfrmPreferences.chbStyleClick(Sender: TObject);
var
  chb: TCheckbox;
  fs: TFontStyle;
begin
  if m_sha = nil then exit;
  chb := Sender as TCheckbox;
  fs := fsBold;
  if chb = chbUnderline then fs := fsUnderline;
  if chb = chbItalic then fs := fsItalic;
  if chb = chbStrikeout then fs := fsStrikeout;
  if chb.Checked then m_sha.Style := m_sha.Style + [fs] else m_sha.Style := m_sha.Style - [fs];
end;

procedure TfrmPreferences.btnForegroundColorClick(Sender: TObject);
begin
  if m_sha <> nil then cld.Color := m_sha.Foreground;
  if cld.Execute then begin
    if m_sha <> nil then m_sha.Foreground := cld.Color;
    ShowHighlighterAttributes;
  end;
end;

procedure TfrmPreferences.btnBackgroundColorClick(Sender: TObject);
begin
  if m_sha <> nil then cld.Color := m_sha.Background;
  if cld.Execute then begin
    if m_sha <> nil then m_sha.Background := cld.Color;
    ShowHighlighterAttributes;
  end;
end;

procedure TfrmPreferences.FormCreate(Sender: TObject);
var
  i, nPos: Integer;
  PropList: TPropList;
  pInfo: PPropInfo;
  obj: TObject;
  sName: String;
begin
  m_Selection := TSelection.Create;
  m_RightEdge := TRightEdge.Create;
  m_ReadOnlyEditorMode := TReadOnlyEditorMode.Create;
  m_ReadOnlyEditorMode.ForegroundColor := clBlack;
  m_ReadOnlyEditorMode.BackgroundColor := clBtnFace;
  m_PassiveConsole := TPassiveConsole.Create;
  m_ErrorLine := TSpecialLine.Create;
  m_ErrorLine.ForegroundColor := clWhite;
  m_ErrorLine.BackgroundColor := clMaroon;
  m_BreakpointLine := TSpecialLine.Create;
  m_BreakpointLine.ForegroundColor := clWhite;
  m_BreakpointLine.BackgroundColor := clRed;
  m_ExecutionLine := TSpecialLine.Create;
  m_ExecutionLine.ForegroundColor := clWhite;
  m_ExecutionLine.BackgroundColor := clNavy;


  for i := 0 to GetPropList(MainForm.SynPerl2.ClassInfo, tkAny, @PropList) - 1 do begin
    pInfo := PropList[i];
    if pInfo^.PropType^.Kind = tkClass then begin
      obj := GetObjectProp(MainForm.SynPerl2, pInfo);
      if (obj <> nil) and (obj is TSynHighlighterAttributes) then begin
        nPos := Pos('attr', LowerCase(pInfo^.Name));
        if nPos > 0 then sName := copy(pInfo^.Name, 1, nPos - 1) else sName := pInfo^.Name;
        if Pos('space', LowerCase(sName)) <> 1 then lbxElements.Items.AddObject(sName, obj);
      end;
    end;
  end;

  m_sHighlighterDefault := HighlighterAsString;
  lbxElements.ItemIndex := 0;
  m_sha := TSynHighlighterAttributes(lbxElements.Items.Objects[0]);
  m_sBasePath := ExtractFilePath(Application.ExeName) + 'Help\';
  for i := 0 to Screen.Fonts.Count-1 do
    cbxFontsAndColors_FontName.Items.Add(Screen.Fonts.Strings[i]);
  eltFontsAndColors.Items[0].Focused := True;
  // Store default fonts and colors
  m_sFontsAndColorsDefault := FontsAndColorsAsString;
end;

procedure TfrmPreferences.btnResetToDefaultClick(Sender: TObject);
begin
  HighlighterAsString := m_sHighlighterDefault;
  ShowHighlighterAttributes;
end;

procedure TfrmPreferences.btnClearForegroundClick(Sender: TObject);
begin
  if m_sha <> nil then m_sha.Foreground := clNone;
  ShowHighlighterAttributes;
end;

procedure TfrmPreferences.btnClearBackgroundClick(Sender: TObject);
begin
  if m_sha <> nil then m_sha.Background := clNone;
  ShowHighlighterAttributes;
end;



procedure TfrmPreferences.eltHelpDblClick(Sender: TObject);
var
  pt: TPoint;
  ItemPart: TSTItemPart;
  nHitColumn: Integer;
  it: TElTreeItem;
  sDirectory: String;
begin
  GetCursorPos(pt);
  pt := eltHelp.ScreenToClient(pt);
  it := eltHelp.GetItemAt(pt.x, pt.y, ItemPart, nHitColumn);
  if (it <> nil) then begin
    case nHitColumn of
      1: begin // Base directory
        m_bRebuildKeywordIndex := True;
        m_bRebuildContents := True;
        if SelectDirectory('Select Help Location', '', sDirectory) then begin
          if it.Text = it.SubItems[0] then it.Text := sDirectory;
          it.SubItems[0] := sDirectory;
          it.SubItems[1] := 'True';  // Contents
          it.SubItems[2] := 'False'; // Keyword Index
          it.SubItems[3] := 'False'; // Full-Text Search
          it.SubItems[4] := '';      // Keyword index file
          it.SubItems[5] := '';      // Full-Text search file
        end;
      end;
      2: begin // Contents
        m_bRebuildContents := True;
        if it.SubItems[1] = 'False' then begin
          it.SubItems[1] := 'True';
        end else begin
          it.SubItems[1] := 'False';
        end;
      end;
      3: begin // Keyword Index
        m_bRebuildKeywordIndex := True;
        if it.SubItems[2] = 'False' then begin
          it.SubItems[2] := 'True';
          it.SubItems[4] := GetNewFileName('KWI', '.txt');
        end else begin
          it.SubItems[2] := 'False';
        end;
      end;
      4: begin // Full-Text Search
        if it.SubItems[3] = 'False' then begin
          it.SubItems[3] := 'True';
          it.SubItems[5] := GetNewFileName('FTS', '.txt');
        end else begin
          it.SubItems[3] := 'False';
        end;
      end;
    else
      m_bRebuildContents := True;
      it.EditText;
    end;
  end;
end;


procedure TfrmPreferences.miAddHelpLocationClick(Sender: TObject);
var
  elt: TElTreeItem;
  sDirectory: String;
begin
  if SelectDirectory('Select Help Location', '', sDirectory) then begin
    m_bRebuildContents := True;
    elt := eltHelp.Items.AddItem(nil);
    elt.ImageIndex := 39;
    elt.ForceButtons := True;
    elt.Text := sDirectory;
    elt.SubItems.Add(sDirectory);
    elt.SubItems.Add('True');  // Contents
    elt.SubItems.Add('False'); // Keyword Index
    elt.SubItems.Add('False'); // Full-Text Search
    elt.SubItems.Add('');      // Keyword index file
    elt.SubItems.Add('');      // Full-Text search file
  end;
end;

procedure TfrmPreferences.miDeleteHelpLocationClick(Sender: TObject);
var
  elt: TElTreeItem;
begin
  elt := eltHelp.Selected;
  if elt <> nil then begin
    m_bRebuildKeywordIndex := m_bRebuildKeywordIndex or (elt.SubItems[2] = 'True');
    elt.Delete;
  end;
end;

procedure TfrmPreferences.btnCancelClick(Sender: TObject);
begin
  if eltHelp.IsEditing then begin
    eltHelp.EndEdit(True);
    ModalResult := mrNone;
  end;
end;

procedure TfrmPreferences.FormDestroy(Sender: TObject);
begin
  if m_slKeywordIndexFiles <> nil then m_slKeywordIndexFiles.Free;
  m_BreakpointLine.Free;
  m_ErrorLine.Free;
  m_ExecutionLine.Free;
  m_PassiveConsole.Free;
  m_ReadOnlyEditorMode.Free;
  m_RightEdge.Free;
  m_Selection.Free;
end;

procedure TfrmPreferences.eltHelpDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  it: TElTreeItem;
  itSel: TElTreeItem;
begin
  it := eltHelp.GetItemAtY(y);
  itSel := eltHelp.Selected;
  if (it <> nil) and (itSel <> nil) then begin
    itSel.MoveToIns(nil, it.VisIndex);
    m_bRebuildContents := True;
    m_bRebuildKeywordIndex := True;
  end;
end;

procedure TfrmPreferences.eltHelpDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := True;
end;

procedure TfrmPreferences.ShowFontsAndColorsItem(it: TElTreeItem);
var
  s: String;
begin
  lbFontsAndColorsItem_Caption.Caption := ' ' + it.Text;
  s := UpperCase(it.Text);
  if s = 'EDITOR WINDOW' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(SynEdit);
  end else if s = 'GUTTER' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(SynEdit.Gutter);
  end else if s = 'READONLY MODE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_ReadOnlyEditorMode);
  end else if s = 'BREAKPOINT LINE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_BreakpointLine);
  end else if s = 'EXECUTION LINE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_ExecutionLine);
  end else if s = 'ERROR LINE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_ErrorLine);
  end else if s = 'RIGHT EDGE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_RightEdge);
  end else if s = 'SELECTION' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_Selection);
  end else if s = 'BREAKPOINTS' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmBreakpoints.eltBreakpoints);
  end else if s = 'CALL STACK' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmCallStack.eltCallStack);
  end else if s = 'VARIABLES' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmVariables.eltWatches);
  end else if s = 'MODULES' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmModules.eltModules);
  end else if s = 'CONSOLE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmConsole.memoConsole);
  end else if s = 'PASSIVE CONSOLE' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(m_PassiveConsole);
  end else if s = 'START PARAMETER' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmConsole.edtParams);
  end else if s = 'ERROR OUTPUT' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmError.memoErrorOutput);
  end else if s = 'CONTENTS' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmHelpContents.eltHelp);
  end else if s = 'INDEX' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmHelpIndex.edtKeyword);
  end else if s = 'KEYWORD LISTBOX' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmHelpIndex.lbxKeywords);
  end else if s = 'COMMUNICATION LOG' then begin
    m_CurrentFontsAndColorsControl := TFontsAndColorsControl.Create(frmCommunicationLog.memoCommunicationLog);
  end else begin
    m_CurrentFontsAndColorsControl := nil;
  end;
  m_bLoadFontsAndColorsItemSettings := True;
  try
    ShowFontsAndColorsItem_Settings;
    ShowFontsAndColorsItem_Preview;
  finally
    m_bLoadFontsAndColorsItemSettings := False;
  end;

  pnlFontsAndColors_Element.Visible := m_CurrentFontsAndColorsControl <> nil;
  pnlFontsAndColors_FontStyle.Visible := (s <> 'EDITOR WINDOW')
    and (s <> 'GUTTER') and (s <> 'BREAKPOINT LINE') and (s <> 'READONLY MODE')
    and (s <> 'EXECUTION LINE') and (s <> 'ERROR LINE')
    and (s <> 'PASSIVE CONSOLE') and (s <> 'RIGHT EDGE') and (s <> 'SELECTION');
  pnlFontsAndColors_Foreground.Visible := (s <> 'GUTTER');
  pnlFontAndColors_Font.Visible := (s <> 'GUTTER') and (s <> 'BREAKPOINT LINE')
    and (s <> 'EXECUTION LINE') and (s <> 'ERROR LINE')
    and (s <> 'PASSIVE CONSOLE') and (s <> 'READONLY MODE')
    and (s <> 'RIGHT EDGE') and (s <> 'SELECTION');
  pnlFontsAndColors_Backgound.Visible := (s <> 'PASSIVE CONSOLE')
    and (s <> 'RIGHT EDGE');
end;


procedure TfrmPreferences.ShowFontsAndColorsItem_Settings;
var
  idx: Integer;
begin
  if m_CurrentFontsAndColorsControl <> nil then begin
    idx := cbxFontsAndColors_FontName.Items.IndexOf(m_CurrentFontsAndColorsControl.Font.Name);
    if idx > -1 then cbxFontsAndColors_FontName.ItemIndex := idx;
    cbxFontsAndColors_FontSize.Text := IntToStr(m_CurrentFontsAndColorsControl.Font.Size);
    pnlFontsAndColors_FontColor.Color := m_CurrentFontsAndColorsControl.ForegroundColor;
    chbFontsAndColors_FontBold.Checked := fsBold in m_CurrentFontsAndColorsControl.Font.Style;
    chbFontsAndColors_FontItalic.Checked := fsItalic in m_CurrentFontsAndColorsControl.Font.Style;
    chbFontsAndColors_FontUnderline.Checked := fsUnderline in m_CurrentFontsAndColorsControl.Font.Style;
    chbFontsAndColors_FontStrikeOut.Checked := fsStrikeOut in m_CurrentFontsAndColorsControl.Font.Style;
    pnlFontsAndColors_BackgroundColor.Color := m_CurrentFontsAndColorsControl.BackgroundColor;
  end;
end;


procedure TfrmPreferences.ApplyFontsAndColorsItemSettings;
var
  fs: set of TFontStyle;
begin
  if (m_CurrentFontsAndColorsControl <> nil) and (not m_bLoadFontsAndColorsItemSettings) then begin
    m_CurrentFontsAndColorsControl.Font.Name := cbxFontsAndColors_FontName.Text;
    m_CurrentFontsAndColorsControl.Font.Size := StrToIntDef(cbxFontsAndColors_FontSize.Text, 8);
    m_CurrentFontsAndColorsControl.ForegroundColor := pnlFontsAndColors_FontColor.Color;
    fs := [];
    if chbFontsAndColors_FontBold.Checked then fs := fs + [fsBold];
    if chbFontsAndColors_FontItalic.Checked then fs := fs + [fsItalic];
    if chbFontsAndColors_FontUnderline.Checked then fs := fs + [fsUnderline];
    if chbFontsAndColors_FontStrikeOut.Checked then fs := fs + [fsStrikeOut];
    m_CurrentFontsAndColorsControl.Font.Style := fs;
    m_CurrentFontsAndColorsControl.BackgroundColor := pnlFontsAndColors_BackgroundColor.Color;
    if MainForm.ActiveMDIChild <> nil then begin
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.Font.Assign(SynEdit.Font);
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.Gutter.Color := SynEdit.Gutter.Color;
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.RightEdgeColor := SynEdit.RightEdgeColor;
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.SelectedColor.Foreground := SynEdit.SelectedColor.Foreground;
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.SelectedColor.Background := SynEdit.SelectedColor.Background;
      if TMDIChild(MainForm.ActiveMDIChild).SynEdit.ReadOnly then begin
        TMDIChild(MainForm.ActiveMDIChild).SynEdit.Font.Color := m_ReadOnlyEditorMode.ForegroundColor;
        TMDIChild(MainForm.ActiveMDIChild).SynEdit.Color := m_ReadOnlyEditorMode.BackgroundColor;
      end else begin
        TMDIChild(MainForm.ActiveMDIChild).SynEdit.Color := SynEdit.Color;
      end;
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.TabWidth := SynEdit.TabWidth;
      TMDIChild(MainForm.ActiveMDIChild).SynEdit.RightEdge := SynEdit.RightEdge;
    end;
  end;
  ShowFontsAndColorsItem_Preview;
end;


procedure TfrmPreferences.ShowFontsAndColorsItem_Preview;
begin
  if m_CurrentFontsAndColorsControl <> nil then begin
    pnlFontsAndColors_Preview.Font := m_CurrentFontsAndColorsControl.Font;
    pnlFontsAndColors_Preview.Font.Color := m_CurrentFontsAndColorsControl.ForegroundColor;
    pnlFontsAndColors_Preview.Color := m_CurrentFontsAndColorsControl.BackgroundColor;
  end;
end;


procedure TfrmPreferences.eltFontsAndColorsKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  it: TElTreeItem;
begin
  it := eltFontsAndColors.Selected;
  if it <> nil then begin
    ShowFontsAndColorsItem(it);
  end;
end;



function TfrmPreferences.InternalGetFontsAndColorsAsString(obj: TObject): String;
var
  sl: TStringList;
  intf: IFontsAndColorsControl;
begin
  intf := TFontsAndColorsControl.Create(obj);
  sl := TStringList.Create;
  sl.Values['Font'] := GetFontAsString(intf.Font);
  sl.Values['ForegroundColor'] := ColorToString(intf.ForegroundColor);
  sl.Values['BackgroundColor'] := ColorToString(intf.BackgroundColor);
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.InternalSetFontsAndColorsAsString(obj: TObject; s: String);
var
  sl: TStringList;
  intf: IFontsAndColorsControl;
begin
  if s <> '' then begin
    intf := TFontsAndColorsControl.Create(obj);
    sl := TStringList.Create;
    sl.CommaText := s;
    SetFontAsString(intf.Font, sl.Values['Font']);
    intf.ForegroundColor := StringToColor(sl.Values['ForegroundColor']);
    intf.BackgroundColor := StringToColor(sl.Values['BackgroundColor']);
    sl.Free;
  end;
end;


function TfrmPreferences.GetFontAsString(AFont: TFont): String;
var
  sl: TStringList;

  function FontStyleToString(fs: TFontStyles): String;
  begin
    Result := '';
    if fsBold in fs then Result := Result + 'Bold,';
    if fsItalic in fs then Result := Result + 'Italic,';
    if fsUnderline in fs then Result := Result + 'Underline,';
    if fsStrikeOut in fs then Result := Result + 'Strikeout,';
    Result := copy(Result, 1, Length(Result) - 1);
  end;

begin
  sl := TStringList.Create;
  sl.Add('Name=' + AFont.Name);
  sl.Add('Size=' + IntToStr(AFont.Size));
  sl.Add('Color=' + ColorToString(AFont.Color));
  sl.Add('Style=' + FontStyleToString(AFont.Style));
  Result := sl.CommaText;
  sl.Free;
end;


procedure TfrmPreferences.SetFontAsString(AFont: TFont; s: String);
var
  sl: TStringList;

  function StringToFontStyle(s: String): TFontStyles;
  begin
    Result := [];
    s := UpperCase(s);
    if Pos('BOLD', s) > 0 then Result := Result + [fsBold];
    if Pos('ITALIC', s) > 0 then Result := Result + [fsItalic];
    if Pos('UNDERLINE', s) > 0 then Result := Result + [fsUnderline];
    if Pos('STRIKEOUT', s) > 0 then Result := Result + [fsStrikeout];
  end;

begin
  sl := TStringList.Create;
  try
    sl.CommaText := s;
    if sl.Values['Name'] <> '' then AFont.Name := sl.Values['Name'];
    AFont.Size := StrToIntDef(sl.Values['Size'], AFont.Size);
    if sl.Values['Color'] <> '' then try AFont.Color := StringToColor(sl.Values['Color']); except end;
    AFont.Style := StringToFontStyle(sl.Values['Style']);
  finally
    sl.Free;
  end;
end;






procedure TfrmPreferences.chbFontsAndColors_FontBoldClick(Sender: TObject);
begin
  ApplyFontsAndColorsItemSettings;
end;

procedure TfrmPreferences.cbxFontsAndColors_FontSizeChange(
  Sender: TObject);
begin
  ApplyFontsAndColorsItemSettings;
end;

procedure TfrmPreferences.btnFontsAndColors_ForegroundColorClick(
  Sender: TObject);
begin
  cld.Color := pnlFontsAndColors_FontColor.Color;
  if cld.Execute then begin
    pnlFontsAndColors_FontColor.Color := cld.Color;
    ApplyFontsAndColorsItemSettings;
  end;
end;

procedure TfrmPreferences.btnFontsAndColors_BackgroundColorClick(
  Sender: TObject);
begin
  cld.Color := pnlFontsAndColors_BackgroundColor.Color;
  if cld.Execute then begin
    pnlFontsAndColors_BackgroundColor.Color := cld.Color;
    ApplyFontsAndColorsItemSettings;
  end;
end;

procedure TfrmPreferences.tsFontsAndColorsShow(Sender: TObject);
begin
  if eltFontsAndColors.Selected <> nil then begin
    ShowFontsAndColorsItem(eltFontsAndColors.Selected);
  end;
end;

procedure TfrmPreferences.eltFontsAndColorsMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  it: TElTreeItem;
begin
  it := eltFontsAndColors.Selected;
  if it <> nil then begin
    ShowFontsAndColorsItem(it);
  end;
end;

procedure TfrmPreferences.btnResetFontsAndColorsToDefaultClick(Sender: TObject);
begin
  FontsAndColorsAsString := m_sFontsAndColorsDefault;
  m_bLoadFontsAndColorsItemSettings := True;
  try
    ShowFontsAndColorsItem_Settings;
    ShowFontsAndColorsItem_Preview;
  finally
    m_bLoadFontsAndColorsItemSettings := False;
  end;
end;

procedure TfrmPreferences.edtTabWidthChange(Sender: TObject);
begin
  SynEdit.TabWidth := udTabWidth.Position;
end;

procedure TfrmPreferences.edtRightEdgeChange(Sender: TObject);
begin
  SynEdit.RightEdge := udRightEdge.Position;
end;

function TfrmPreferences.GetRightEdge: Integer;
begin
  Result := udRightEdge.Position;
end;

function TfrmPreferences.GetTabWidth: Integer;
begin
  Result := udTabWidth.Position;
end;

procedure TfrmPreferences.SetRightEdge(const Value: Integer);
begin
  udRightEdge.Position := Value;
end;

procedure TfrmPreferences.SetTabWith(const Value: Integer);
begin
  udTabWidth.Position := Value;
end;

end.



