{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_Variables.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is Jrgen Gntherodt.

Portions created by Jrgen Gntherodt <jguentherodt@users.sourceforge.net>
are Copyright (C) 2001-2002 Jrgen Gntherodt. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_Variables.pas,v 1.7 2002/05/26 13:47:34 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE webpage,
located at http://open-perl-ide.sourceforge.net or http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_Variables;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, SynEdit, ElTree, ElHeader, DockPanel,
  message_interface;

  
type
  TfrmVariables = class(TDockableForm)
    pnlClient: TPanel;
    Splitter1: TSplitter;
    pnlLocalVariables: TPanel;
    Panel3: TPanel;
    memoLocals: TMemo;
    pnlWatches: TPanel;
    Panel1: TPanel;
    eltWatches: TElTree;
    tmrExpandArray: TTimer;
    procedure eltWatchesDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure eltWatchesDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure eltWatchesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure eltWatchesResize(Sender: TObject);
    procedure eltWatchesHeaderColumnResize(Sender: TObject;
      SectionIndex: Integer);
    procedure eltWatchesDblClick(Sender: TObject);
    procedure eltWatchesValidateInplaceEdit(Sender: TObject;
      Item: TElTreeItem; Section: TElHeaderSection; var Text: String;
      var Accept: Boolean);
    procedure eltWatchesItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure eltWatchesTryEdit(Sender: TObject; Item: TElTreeItem;
      Section: TElHeaderSection; var CellType: TFieldTypes;
      var CanEdit: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure eltWatchesItemDeletion(Sender: TObject; Item: TElTreeItem);
    procedure eltWatchesItemLoad(Sender: TObject; Stream: TStream;
      Item: TElTreeItem);
    procedure tmrExpandArrayTimer(Sender: TObject);
  private
    { Private declarations }
    m_slItemLookup: TStringList;
    m_eltNew: TElTreeItem;
    m_item: TElTreeItem;
    function InternalParseTypeValue(const sTypeValue: String; out sType, sValue: String): Boolean;
    procedure ParseTypeValue(Item: TElTreeItem; sTypeValue: String);
    function GetDataAsString: String;
    procedure SetDataAsString(s: String);
    procedure RefreshItems(Item: TElTreeItem);
    procedure UpdateMembers(item: TElTreeItem);
    procedure ExpandArrayItem(item, itemToDelete: TElTreeItem);
    procedure ExpandHashItem(item: TElTreeItem);
    procedure ExpandClassItem(item: TElTreeItem);
    procedure ExpandItem(item: TElTreeItem);
  public
    { Public declarations }
    procedure ClearWatchTree;
    procedure UpdateVariable(sRequestID, sName, sValue: String);
    procedure EvaluateVariable(Item: TElTreeItem; sText: String);
    function HasMembers(Item: TElTreeItem): Boolean;
    function ExtractVariableFromCode(sCode: String): String;
    function GetNextHexDecode(var lpszEncodedString: PChar): String;
    property DataAsString: String Read GetDataAsString Write SetDataAsString;
  end;

var
  frmVariables: TfrmVariables;

implementation

uses Main;

{$R *.DFM}


procedure TfrmVariables.ClearWatchTree;
var
  i: Integer;
  it: TElTreeItem;
begin
  eltWatches.IsUpdating := True;
  try
    eltWatches.FullCollapse;
    for i := 0 to eltWatches.Items.VisCount - 1 do begin
      it := eltWatches.Items.ItemAsVis[i];
      if it <> nil then it.Clear;
    end;
  finally
    eltWatches.IsUpdating := False;
  end;
end;


procedure TfrmVariables.ExpandItem(item: TElTreeItem);
begin
  if Pos('ARRAY', UpperCase(Item.SubItems[0])) > 0 then begin
    item.Clear;
    ExpandArrayItem(item, nil);
  end else if Pos('HASH', UpperCase(Item.SubItems[0])) > 0 then begin
    ExpandHashItem(item);
  end else if Pos('CLASS', UpperCase(Item.SubItems[0])) > 0 then begin
    ExpandClassItem(item);
  end else if Item.SubItems[0] = '' then begin
    // <...> - array element found
    if item.Parent <> nil then begin
      item.Clear;
      item.Text := '<eval...>';
      m_item := item;
      tmrExpandArray.Enabled := True;
    end;
  end;
end;


procedure TfrmVariables.ExpandArrayItem(item, itemToDelete: TElTreeItem);
var
  sPath: String;
  lpszResponse: PChar;
  sName, sTypeValue: String;
  nCount: Integer;
  ti: TElTreeItem;
  nMinIndex: Integer;
begin
  nMinIndex := item.Count;
  if itemToDelete <> nil then dec(nMinIndex);
  sPath := Item.GetFullNameEx('->', False);

  lpszResponse := PChar(MainForm.BlockingEval(Format(
      '%d'#13#10'LongEval(\%s,%d,%d)', [Longint(Item), sPath, nMinIndex, nMinIndex + 1025 - 1])));

  eltWatches.IsUpdating := True;

  try
    nCount := 0;

    // There are 1025 items requested but only 1024 shown.
    sName := GetNextHexDecode(lpszResponse);
    while (sName <> '') do begin
      inc(nCount);
      if nCount < 1025 then begin
        if itemToDelete <> nil then begin
          ti := itemToDelete;
          itemToDelete := nil;
          ti.Clear;
          ti.SubItems.Clear;
          ti.Text := sName;
        end else begin
          ti := eltWatches.Items.AddChild(Item, sName);
          m_slItemLookup.AddObject(IntToStr(Longint(ti)), ti);
        end;
        sTypeValue := GetNextHexDecode(lpszResponse);
        ParseTypeValue(ti, sTypeValue);
        UpdateMembers(ti);
        sName := GetNextHexDecode(lpszResponse);
      end else break;
    end;

    if nCount > 1024 then begin
      ti := eltWatches.Items.AddChild(Item, '<...>');
      m_slItemLookup.AddObject(IntToStr(Longint(ti)), ti);
      ti.SubItems.Add('');
      ti.SubItems.Add('');
      UpdateMembers(ti);
    end;

  finally
    eltWatches.IsUpdating := False;
  end;
end;


procedure TfrmVariables.ExpandHashItem(item: TElTreeItem);
var
  sPath: String;
  lpszResponse: PChar;
  sName, sTypeValue: String;
  ti: TElTreeItem;
begin
  sPath := Item.GetFullNameEx('->', False);
  Item.Clear;

  lpszResponse := PChar(MainForm.BlockingEval(Format(
    '%d'#13#10'LongEval(\%s,%d,%d)', [Longint(Item), sPath, 0, 1023])));

  sName := GetNextHexDecode(lpszResponse);
  while (sName <> '') do begin
    ti := eltWatches.Items.AddChild(Item, sName);
    m_slItemLookup.AddObject(IntToStr(Longint(ti)), ti);
    sTypeValue := GetNextHexDecode(lpszResponse);
    ParseTypeValue(ti, sTypeValue);
    UpdateMembers(ti);
    sName := GetNextHexDecode(lpszResponse);
  end;
end;


procedure TfrmVariables.ExpandClassItem(item: TElTreeItem);
begin
  ExpandHashItem(item);
end;


procedure TfrmVariables.RefreshItems(Item: TElTreeItem);
var
  i: Integer;
begin
  eltWatches.IsUpdating := True;
  Item.SubItems.Clear;
  Item.Collapse(True);
  Item.Clear;
  eltWatches.IsUpdating := False;
  EvaluateVariable(Item, Item.GetFullNameEx('->', False));
  if Item.Expanded then begin
    for i := 0 to Item.Count - 1 do RefreshItems(Item.Item[i]);
  end;
end;


function TfrmVariables.GetDataAsString: String;
var
  ms: TMemoryStream;
begin
  ms := TMemoryStream.Create;
  try
    try eltWatches.Items.SaveToStream(ms); except end;
    ms.Seek(0, soFromBeginning);
    SetLength(Result, 2 * ms.Size);
    BinToHex(PChar(ms.Memory), PChar(Result), ms.Size);
  finally
    ms.Free;
  end;
end;


procedure TfrmVariables.SetDataAsString(s: String);
var
  ms: TMemoryStream;
begin
  eltWatches.Items.Clear;
  m_eltNew := eltWatches.Items.AddItem(nil);
  if s = '' then exit;
  ms := TMemoryStream.Create;
  try
    ms.Size := Length(s) div 2;
//    HexToBin(PChar(s), PChar(ms.Memory), ms.Size);
    HexToBin(PChar(s), PChar(ms.Memory), Length(s));
    ms.Seek(0, soFromBeginning);
    try eltWatches.Items.LoadFromStream(ms); except eltWatches.Items.Clear; end;
  finally
    if eltWatches.Items.Count > 0 then begin
      // NewItem either is first or last item
      if eltWatches.Items[0].Text = ''
        then m_eltNew := eltWatches.Items[0]
        else m_eltNew := eltWatches.Items[eltWatches.Items.Count - 1];
    end else begin
      m_eltNew := eltWatches.Items.AddItem(nil);
    end;
    m_eltNew.Text := '';
    m_eltNew.SubItems.Clear;
    m_eltNew.Clear;
    ms.Free;
  end;
end;


function TfrmVariables.GetNextHexDecode(var lpszEncodedString: PChar): String;
var
  sLength: String;
  nLength: Integer;
begin
  SetLength(sLength, 8);
  Move(lpszEncodedString^, sLength[1], 8);
  inc(lpszEncodedString, 8);
  nLength := StrToIntDef('$' + sLength, -1);
  if nLength > -1 then begin
    SetLength(Result, nLength);
    Move(lpszEncodedString^, Result[1], nLength);
    inc(lpszEncodedString, nLength);
  end else begin
    Result := '';
    lpszEncodedString := '';
  end;
end;


function TfrmVariables.ExtractVariableFromCode(sCode: String): String;
var
  iPos: Integer;
begin
  iPos := Pos('(\', sCode);
  if iPos > 0 then begin
    sCode := copy(sCode, iPos + 2, Length(sCode));
    if copy(sCode, Length(sCode), 1) = ')' then begin
      sCode := copy(sCode, 1, Length(sCode) - 1);
    end;
  end;
  Result := sCode;
end;


function TfrmVariables.HasMembers(Item: TElTreeItem): Boolean;
begin
  Result := (Pos('ARRAY', UpperCase(Item.SubItems[0])) > 0)
         or (Pos('HASH', UpperCase(Item.SubItems[0])) > 0)
         or (Pos('CLASS', UpperCase(Item.SubItems[0])) > 0)
         or ((Item.SubItems[0] = '') and (Item.Text <> '%'));
end;

function TfrmVariables.InternalParseTypeValue(const sTypeValue: String; out sType, sValue: String): Boolean;
var
  iPos: Integer;
begin
  iPos := Pos(' = ', sTypeValue);
  Result := iPos > 0;
  if Result then begin
    sType := copy(sTypeValue, 1, iPos - 1);
    sValue := copy(sTypeValue, iPos + 3, Length(sTypeValue));
  end;
end;

procedure TfrmVariables.ParseTypeValue(Item: TElTreeItem; sTypeValue: String);
var
  sType, sValue: String;
begin
  if InternalParseTypeValue(sTypeValue, sType, sValue) then begin
    if Item <> nil then begin
      Item.SubItems.Add(sType);
      Item.SubItems.Add(sValue);
    end;
  end else begin
    Item.SubItems.Add(sTypeValue);
    Item.SubItems.Add(sTypeValue);
  end;
end;


procedure TfrmVariables.UpdateMembers(item: TElTreeItem);
var
  bHasMembers: Boolean;
begin
  bHasMembers := HasMembers(item);
  if bHasMembers and (item.Count = 0) then begin
    item.Collapse(True);
    eltWatches.Items.AddChild(item, '<internal item>');
  end;
  if (not bHasMembers) and (item.Count > 0) then begin
    item.Clear;
  end;
end;


procedure TfrmVariables.EvaluateVariable(Item: TElTreeItem; sText: String);
var
  sResponse: String;
begin
  if sText <> '%' then begin
    sResponse := MainForm.BlockingEval(
        IntToStr(Longint(Item)) + #13#10 + 'ShortEval(\' + sText +  ')');
  end else sResponse := '';
  ParseTypeValue(Item, sResponse);
  UpdateMembers(Item);
end;


procedure TfrmVariables.UpdateVariable(sRequestID, sName, sValue: String);
var
  item: TElTreeItem;
  nPos, idx: Integer;
begin
  idx := m_slItemLookup.IndexOf(sRequestID);
  if idx > -1 then begin
    item := TElTreeItem(m_slItemLookup.Objects[idx]);
    item.ColumnText.Clear;
    // Delete all children to enforce re-evaluation on expanding.
    if not item.Expanded then item.Clear;
    ParseTypeValue(item, sValue);
    UpdateMembers(item);
  end else begin
    nPos := Pos('/', sRequestID);
    if nPos > 0 then begin
      idx := m_slItemLookup.IndexOf(copy(sRequestID, 1, nPos - 1));
      if idx > -1 then begin
        item := TElTreeItem(m_slItemLookup.Objects[idx]);
        item := eltWatches.Items.AddChild(item, copy(sRequestID, nPos + 1, MAXINT));
        m_slItemLookup.AddObject(IntToStr(Longint(item)), item);
        ParseTypeValue(item, sValue);
        UpdateMembers(item);
      end;
    end;
  end;
end;


procedure TfrmVariables.eltWatchesDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TSynEdit;
end;

procedure TfrmVariables.eltWatchesDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  eltItem: TElTreeItem;
begin
  if Source is TSynEdit then begin
    m_eltNew.Delete;
    try
      eltItem := eltWatches.Items.AddItem(nil);
      m_slItemLookup.AddObject(IntToStr(Longint(eltItem)), eltItem);
      eltItem.Text := TSynEdit(Source).SelText;
      EvaluateVariable(eltItem, eltItem.Text);
    finally
      m_eltNew := eltWatches.Items.AddItem(nil);
    end;
  end;
end;

procedure TfrmVariables.eltWatchesKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  bExpanded: Boolean;
  it: TElTreeItem;
begin
  if eltWatches.ItemFocused = nil then exit;
  case Key of
    13: begin
      if eltWatches.ItemFocused <> nil then RefreshItems(eltWatches.ItemFocused);
    end;
    46: begin
      if eltWatches.ItemFocused <> nil then begin
        if (eltWatches.ItemFocused.Level = 0) and (eltWatches.ItemFocused <> m_eltNew)
          then eltWatches.ItemFocused.Delete;
      end;
    end;
   116: begin
     if ssCtrl in Shift then begin
     end else begin
       it := eltWatches.ItemFocused;
       if it <> nil then begin
         bExpanded := it.Expanded;
         RefreshItems(it);
         if bExpanded then it.Expand(False);
       end;
     end;
   end;
  else
  end;
end;

procedure TfrmVariables.eltWatchesResize(Sender: TObject);
begin
  if eltWatches.IsEditing then eltWatches.EndEdit(True);
  eltWatches.HeaderSections[2].Width := eltWatches.ClientWidth
    - eltWatches.HeaderSections[0].Width
    - eltWatches.HeaderSections[1].Width;
end;

procedure TfrmVariables.eltWatchesHeaderColumnResize(Sender: TObject;
  SectionIndex: Integer);
begin
  if eltWatches.IsEditing then eltWatches.EndEdit(True);
  eltWatches.HeaderSections[2].Width := eltWatches.ClientWidth
    - eltWatches.HeaderSections[0].Width
    - eltWatches.HeaderSections[1].Width;
end;

procedure TfrmVariables.eltWatchesDblClick(Sender: TObject);
var
  pt: TPoint;
  ItemPart: TSTItemPart;
  nHitColumn: Integer;
  it: TElTreeItem;
begin
  GetCursorPos(pt);
  pt := eltWatches.ScreenToClient(pt);
  it := eltWatches.GetItemAt(pt.x, pt.y, ItemPart, nHitColumn);
  if (it <> nil) and (nHitColumn = 0) then it.EditText;
end;

procedure TfrmVariables.eltWatchesValidateInplaceEdit(Sender: TObject;
  Item: TElTreeItem; Section: TElHeaderSection; var Text: String;
  var Accept: Boolean);
var
  sResponse: String;
begin
  Accept := Accept and (Text <> '');
  if (not Accept) then exit;
  if Section.Index = 0 then begin
    // Evaluate watch
    Item.SubItems.Clear;
    Item.Collapse(True);
    Item.Clear;
    EvaluateVariable(Item, Text);
    Item.Text := Text;
    if Item = m_eltNew then m_eltNew := eltWatches.Items.AddItem(nil);
  end else begin
    // Set watch to new value
    sResponse := MainForm.BlockingEval(
        IntToStr(Longint(Item)) + #13#10
      + Item.GetFullNameEx('->', False) + '=' + Text);
    Item.SubItems[1] := sResponse;
  end;
end;

procedure TfrmVariables.eltWatchesItemExpand(Sender: TObject;
  Item: TElTreeItem);
begin
  if (Item.ChildrenCount = 1) and (Item.Children[0].Text = '<internal item>') then begin
    ExpandItem(Item);
  end;
end;


procedure TfrmVariables.eltWatchesTryEdit(Sender: TObject;
  Item: TElTreeItem; Section: TElHeaderSection; var CellType: TFieldTypes;
  var CanEdit: Boolean);
begin
  if Section.Index = 0 then CanEdit := Item.Parent = nil;
  if Section.Index = 2 then CanEdit := Item.SubItems[0] = 'scalar';
end;


procedure TfrmVariables.FormCreate(Sender: TObject);
begin
  m_eltNew := eltWatches.Items.AddItem(nil);
  m_slItemLookup := TStringList.Create;
  m_slItemLookup.Sorted := True;
end;


procedure TfrmVariables.FormDestroy(Sender: TObject);
begin
  m_slItemLookup.Free;
end;


procedure TfrmVariables.eltWatchesItemDeletion(Sender: TObject;
  Item: TElTreeItem);
var
  idx: Integer;
begin
  if csDestroying in ComponentState then exit;
  idx := m_slItemLookup.IndexOf(IntToStr(Longint(Item)));
  if idx > -1 then m_slItemLookup.Delete(idx);
end;

procedure TfrmVariables.eltWatchesItemLoad(Sender: TObject;
  Stream: TStream; Item: TElTreeItem);
begin
  m_slItemLookup.AddObject(IntToStr(Longint(Item)), Item);
end;

procedure TfrmVariables.tmrExpandArrayTimer(Sender: TObject);
begin
  tmrExpandArray.Enabled := False;
  ExpandArrayItem(m_item.Parent, m_item);
end;

end.
